/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
* File Name          : CamInt_x24.c
* Author             : AST Robotics group
* Date First Issued  : 20 March 2007 : Version 1.0
* Description        : header file for CamInt_x24.c
********************************************************************************
* History:
* 28 May 2007 : Version 1.2
* 11 May 2007 : Version 1.1
* 20 March 2007:Version 1.0
*******************************************************************************
 THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS WITH
 CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
 AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
 OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
 OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
 CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

/* Define to prevent recursive inclusion ------------------------------------ */
#ifndef __CamInt_x24_H
#define __CamInt_x24_H

/* Includes ------------------------------------------------------------------*/
#include "91x_lib.h"
//#include "definitions.h"

/* CODE ----------------------------------------------------------------------*/

/* Exported types ------------------------------------------------------------*/
#define x24_MaxBlocks 16

typedef struct {
//  vu8 Flag;
  vu32 CapPartial;
  vu32 CapGlobal;
  u32  OldGlobal;
} TTimeStamp;

typedef volatile enum { 
  x24_StateWaitForFrame,
  x24_StateAcquisitionStart,
  x24_StateAcquisitionRows,
  x24_StateAcquisitionEnd,
  x24_StateError=50
} Tx24_ImgState;

typedef volatile enum { 
  Tx24_BlockOwner_Free,
  Tx24_BlockOwner_Engine,
  Tx24_BlockOwner_User,
  Tx24_BlockOwner_Error=40
} Tx24_BlockOwner;

typedef struct {
  u32 SrcAdd;    /* The current source address (byte-aligned) of the data to be transferred.*/
  u32 DesAdd;    /* The current destination address (byte-aligned) of the data to be transferred.*/
  u32 LLstItm;   /* The word- aligned address for the next Linked List Item. */
  u32 CCR;       /* The current Channel Control Register */
} Tx24_DMA_LLI;

typedef struct {
  vu8         	    *pData;
  Tx24_BlockOwner   Owner;
  vu16		    CurrentBlockRow;
  void              *pNext;
} Tx24_ImgBlock;

typedef struct {
  Tx24_ImgState   State;
  vu16            CurrImgRow;
  vu8 	          CurrentBlock;
  vu32            FrameCount;
  RTC_TIME        Time;
  Tx24_ImgBlock   *pFirstBlock;
  void            *pNext;
} Tx24_ImgJob;

typedef struct {
  Tx24_ImgBlock   Block[x24_MaxBlocks];
  
  u16             DeviceID;
  u8              DeviceRev;
  u8              FirmwareVsnMajor;
  u8              FirmwareVsnMinor;
  u8              PatchVsnMajor;
  u8              PatchVsnMinor;
  
  u16             Res_Width;        // 160
  u16             Res_Height;       // 120
  u8              Res_Colour_Type;  // x24_ImageFormatYUV_422
  u8              Res_Colour_Size;  // 2 ( 2 bytes/pixel)
  u16             Row_Size;         // 320 Data_Row=(Res_NColons * Res_Colour_Size)
  u16             Max_ImgRows;      // 120
  u16             Max_BlockRows;
  u8              SyncEnabled;
  u8              SendSyncFC;
  vu16            MasterFrameCount;
  
  Tx24_ImgJob     *pCurrJob;       // Current engine job
  
  Tx24_DMA_LLI    DMA_LLI[2];
  Tx24_DMA_LLI    *pCurrDMA_LLI;
  Tx24_ImgBlock   *pCurrBlock;
  vu8              *pDMA_Buffer;
  vu16            FrameCount;
} Tx24_Engine;

typedef struct {
  u8 *pDataMemory;
  u8  MemBlocksNum;
  u8  ImgBlocksNum;
  u16 ImgBlockSize;
  u16 ImageWidth;
  u16 ImageHeight;
  u8  ImageFormat;
  u8  Zoom16;
  s16 PanH;
  s16 PanV;
  u16 DesiredFrameRate_Num;
  u8  DesiredFrameRate_Den;
  u16 ExtClockFreqMhz_Num;
  u8  ExtClockFreqMhz_Den;
  u8  FrameRateMode;
  u8  SensorMode;
  u8  JPEGSqueezeValue;
  u8  JPEGClockDerate;
  u16 JPEGLineLength;
  float SysClock; // Range 35:270 MHz
  u8  SyncEnabled;
  s8  ExposureCompensation;
} Tx24_InitTypeDef;


/* Exported constants --------------------------------------------------------*/

#define x24_CE x24_CE_GPIO->DR[x24_CE_GPIOx_Pin<<2]

//------- Address of camera on the I2C BUS ------------------
#define x24_I2C_Addr 0x20 // I2C device address of the camera VS6624

//-------------------- VS6624 register address Map ---------------------

//LowLevelControlRegister
#define x24_MicroEnable 0xC003 // 0x02 - enable
#define x24_DIO_Enable  0xC044 // 0x01 - enable

//Video Timing Control
#define bSysClkMode     0x0880 // 0:Default, 1:User
#define fpUserPLLClk    0x0883 // FP900 [6:270] MHz

//DeviceParameters[read only]
#define x24_uwDeviceId        0x0001 // ID 624/724
#define x24_bDeviceRev        0xc052
#define x24_bFirmwareVsnMajor 0x0004
#define x24_bFirmwareVsnMinor 0x0006
#define x24_bPatchVsnMajor    0x0008
#define x24_bPatchVsnMinor    0x000A

//HostInterfaceManagerControl - in all stable states
#define x24_UserCommand 0x0180  // 0x02 - run

//ModeSetup
#define x24_SensorMode 0x0308

//PipeSetupBank0
#define x24_bImageSize0           0x0380
#define x24_uwManualHSize0        0x0383
#define x24_uwManualVSize0        0x0387
#define x24_uwZoomStepHSize0      0x038B
#define x24_uwZoomStepVSize0      0x038F
#define x24_bZoomControl0         0x0392
#define x24_uwPanStepHSize0       0x0395
#define x24_uwPanStepVSize0       0x0399
#define x24_bPanControl0          0x039C
#define x24_bCropControl0         0x039E
#define x24_uwManualCropHStart0   0x03A1
#define x24_uwManualCropHSize0    0x03A5
#define x24_uwManualCropVStart0   0x03A9
#define x24_uwManualCropVSize0    0x03AD
#define x24_bImageFormat0         0x03B0
#define x24_bJpegSqueezeSettings0 0x03C0 // 0:User, 1:Manual, 2:ManualStill
#define x24_uwJpegTargetFileSize0 0x03c3

//PipeSetupBank1
#define x24_Pipe1Offset           0x0080


#define x24_bHiSqueezeValue       0x2508 // 6:Highest, 255:Lowest
#define x24_uwLineLength          0x2511
#define x24_bOIFCLKRatio          0x2514 // Range: [0:6], 0:NoSLower, N:Slower 2*N times
#define x24_uwThres               0x251b

//VideoTimingParameterHostInputs
#define x24_uwExtClockFreqMhz_Num 0x0605
#define x24_bExtClockFreqMhz_Den  0x0608

//StaticFrameRateControl
#define x24_uwDesiredFrameRate_Num  0x0C81
#define x24_bDesiredFrameRate_Den   0x0C84

//AutomaticFrameRateControl
#define x24_bFrameRateMode 0x0D80      //0 manual   1 auto

//OutputFormatterControl
#define x24_bHSyncSetup     0x2390
#define x24_bVSyncSetup     0x2392
#define x24_bPClkSetup      0x23B0
#define x24_bSyncCodeSetup  0x23AE
#define x24_bRgbSetup       0x2382

// Exposure Control
#define x24_biExposureCompensation 0x109C // -25 to 12 in 1/6 EV

#define x24_ViewLiveControl   0x0480

//-------------------- x24 register values table ------------------

// UserCommand
#define x24_UserCommand_RUN      0x02
#define x24_UserCommand_PAUSE    0x03
#define x24_UserCommand_STOP     0x04
#define x24_UserCommand_SNAPSHOT 0x05
#define x24_UserCommand_FLASHGUN 0x06

// SysClkMode
#define SysClkModeDefault        0x00
#define SysClkModeUser           0x01

// SensorMode
#define x24_SensorMode_UXGA      0x00
#define x24_SensorMode_SVGA      0x01

// ImageSize
#define x24_ImageSize_UXGA   0x00 // 1600 x 1200
#define x24_ImageSize_SXGA   0x01 // 1280 x 1024
#define x24_ImageSize_SVGA   0x02 // 1024 x 768
#define x24_ImageSize_VGA    0x03 // 640 x 480
#define x24_ImageSize_CIF    0x04 // 352 x 288
#define x24_ImageSize_QVGA   0x05 // 320 x 240
#define x24_ImageSize_QCIF   0x06 // 176 x 144
#define x24_ImageSize_QQVGA  0x07 // 160 x 120
#define x24_ImageSize_QQCIF  0x08 // 88 x 72
#define x24_ImageSize_Manual 0x09

// ImageFormat
#define x24_ImageFormat_YCbCr_Rec601 0x01
#define x24_ImageFormat_YCbCr_400    0x03
#define x24_ImageFormat_RGB_565      0x04
#define x24_ImageFormat_RGB_444      0x06
#define x24_ImageFormat_Bayer10      0x09
#define x24_ImageFormat_Bayer8       0x0A
#define x24_ImageFormat_JPG          0x0B

#define x24_CropManual 0x00
#define x24_CropAuto   0x01

#define x24_FrameRate_manual 0x00
#define x24_FrameRate_auto   0x01

#define x24_bHSyncSetupValue     0x0F // 0x0F
#define x24_bVSyncSetupValue     0x07 // 0x07
#define x24_bSyncCodeSetupValue  0x00 // 0x01 embedded codes always on
#define x24_bRgbSetupValue       0x08
#define x24_bPClkSetupValue      0x01 // 0x51 PCLK OFF in SyncCode

#define x24_FLAG_SHOOT		0x01
#define x24_FLAG_SYNC		0x02

#define SET_FLAG(Flag)		x24_EngineFlags|=Flag
#define RESET_FLAG(Flag)	x24_EngineFlags&=(~Flag)

/* Exported variables --------------------------------------------------*/

extern volatile u16 DesiredFrameRate;
//extern TTimeStamp x24_Tcam1, x24_Tcam2;
extern u8 x24_I2C_Driver_Handle;
extern volatile u8 x24_EngineFlags;
extern Tx24_Engine x24_Engine;
extern vu32 msTimeGlobal;

/* Exported macro ------------------------------------------------------------*/
#define x24_DMA_ChannelEnable 0x00000001

#define x24_FIQ_HandlerMACRO \
//ENDM x24_FIQ_HandlerMACRO

/* Private functions ---------------------------------------------------------*/
/* Exported functions ------------------------------------------------------- */

void x24_VSync_IRQ_Handler(void);
void x24_DMA_IRQHandler(void);
//void x24_TIM_SYNC_Handler(void);
void x24_I2C_HwConfig(void);
void x24_HwConfig(u32 CamFreqHz);
void x24_DisableCamera(void);
void x24_StructInit(Tx24_InitTypeDef *x24_InitStruct);
void x24_SendUserCommand(u32 Cmd);
void x24_SendPatches(void);
//void SetExtClockRegisters(u16 ExtClockFreqMhz_Num, u8 ExtClockFreqMhz_Den);
void x24_Init(Tx24_InitTypeDef *x24_InitStruct);
void x24_ReInit(Tx24_InitTypeDef *x24_InitStruct);
void x24_GrabFrameStart(Tx24_ImgJob *Job);
int  x24_IsGrabFrameComplete(Tx24_ImgJob *Job);
void x24_GrabFrameWaitComplete(Tx24_ImgJob *Job);
void x24_GrabFrame(Tx24_ImgJob *Job);
int  x24_NRowAcquired(Tx24_ImgJob *Job);
void x24_SetPTZ(Tx24_InitTypeDef *x24_InitStruct);
//void x24_StartContinuousGrab(u8 *Buffer, u32 BufLen);
void DMA_Configuration(void);

//void StartSyncOut(u8 FramerateTime5);
//void StartSyncIn(void);
//void StartTimSync(void);

ErrorStatus x24_WriteRegs(u16 Reg_Addr, u8 *pBuffer, u8 DataLenght);
//ErrorStatus x24_WriteRegs_NoWait(u16 Reg_Addr, u8 *pBuffer, u8 DataLenght);
ErrorStatus x24_WriteReg8(u16 Reg_Addr, u8 Data);
ErrorStatus x24_WriteReg16(u16 Reg_Addr, u16 Data);
//ErrorStatus x24_WriteReg16_Wait(u16 Reg_Addr, u16 Data, u8 wait);
ErrorStatus x24_WriteRegF900(u16 Reg_Addr, float f);
void x24_WriteReg8n(const u16 Regs[][2], u32 dim);

ErrorStatus x24_ReadRegs(u16 Reg_Addr, u8 *pBuffer, u8 DataLenght);
//#define x24_ReadReg8(Reg_Addr, Data8) x24_ReadRegs(Reg_Addr, Data8, 1)
//#define x24_ReadReg8(Reg_Addr, Data8) i2c_read_reg(Reg_Addr, Data8, 1)
ErrorStatus x24_ReadReg8(u16 Reg_Addr, u8 *Data8);
ErrorStatus x24_ReadReg16(u16 Reg_Addr, u16 *Data16);

int MSTimeOut(u32 nCount, u32 *Start, bool Initialize);
u16 float_to_f900(float f);
float f900_to_float(u16 f900);

#endif /* __CamInt_x24_H */

/******************* (C) COPYRIGHT 2007 STMicroelectronics *****END OF FILE****/
