/*******************************************************************************
 *                            C STMicroelectronics
 *    Reproduction and Communication of this document is strictly prohibited
 *      unless specifically authorized in writing by STMicroelectronics.
 *------------------------------------------------------------------------------
 *                              Imaging Division
 *------------------------------------------------------------------------------
 * $Id: c4mHaarGetFeatureValue_R.h.rca 1.2 Wed Jul 23 10:36:09 2008 vitor.schwambach@st.com Experimental $
 *------------------------------------------------------------------------------
 * $Log: c4mHaarGetFeatureValue_R.h.rca $
 * 
 *  Revision: 1.2 Wed Jul 23 10:36:09 2008 vitor.schwambach@st.com
 *  No modification.
 * 
 *  Revision: 1.1 Fri Jul 18 13:56:55 2008 vitor.schwambach@st.com
 *  Initial version from AST
 ******************************************************************************/

#ifndef __C4MHAARGETFEATUREVALUE_R_H_
#define __C4MHAARGETFEATUREVALUE_R_H_

#ifndef __C4MTYPES_H_
#include "c4mTypes.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif
	
	
//ms inline it
#include "c4mErrors.h"
#define FEATURE_INLINE	1
	
#if	FEATURE_INLINE	
static inline 
#endif
int8 c4mHaarGetFeatureValue_IUR(uint32 *p_img,uint32 type, uint32 jump_r, uint32 jump_c,
	int32 *result)
#if	!FEATURE_INLINE
;
#else
{
	uint32 *p0_img, *p1_img;
	int32 tmp;
	int8 ret_code = C4M_RET_OK;

	/* The true function start here */	
	tmp = 0;
	p0_img = p_img;
	switch( type ) {
		case 0:
			/* -1 +-----+ +1
			 *    |  -  |
			 * +2 +-----+ -2
			 *    |  +  |
			 * -1 +-----+ +1 */
			p1_img = p0_img + jump_r;
			tmp = (int32)(*p1_img - *p0_img);
			p0_img = p1_img + jump_r;
			tmp -= (int32)(*p0_img - *p1_img);
			/* Go on the next column */
			p0_img = p_img + jump_c;
			p1_img = p0_img + jump_r;
			tmp -= (int32)(*p1_img - *p0_img);
			p0_img = p1_img + jump_r;
			tmp += (int32)(*p0_img - *p1_img);
			break;
		case 1:
			/* +1   -2    +1
			 * +-----+-----+
			 * |  +  |  -  |
			 * +-----+-----+
			 * -1   +2    -1 */
			p1_img = p0_img + jump_r;
			tmp -= (int32)(*p1_img - *p0_img);
			/* Go on the next column */
			p0_img += jump_c;
			p1_img += jump_c;
			tmp += 2*(int32)(*p1_img - *p0_img);
			/* Go on the next column */
			p0_img += jump_c;
			p1_img += jump_c;
			tmp -= (int32)(*p1_img - *p0_img);
			break;
		case 2:
			/* -1   +3    -3    +1
			 * +-----+-----+-----+
			 * |  -  |  +2 |  -  |
			 * +-----+-----+-----+
			 * +1   -3    +3    -1 */
			p1_img = p0_img + jump_r;
			tmp = (int32)(*p1_img - *p0_img);
			/* Go on the next column */
			p0_img += jump_c;
			p1_img += jump_c;
			tmp -= 3*(int32)(*p1_img - *p0_img);
			/* Go on the next column */
			p0_img += jump_c;
			p1_img += jump_c;
			tmp += 3*(int32)(*p1_img - *p0_img);
			/* Go on the next column */
			p0_img += jump_c;
			p1_img += jump_c;
			tmp -= (int32)(*p1_img - *p0_img);
			break;
		case 3:
			/* -1 +-----+ +1
			 *    |  -  |
			 * +3 +-----+ -3
			 *    |  +  |
			 * -3 +-----+ +3
			 *    |  -  |
			 * +1 +-----+ -1 */
			p1_img = p0_img + jump_r;
			tmp = (int32)(*p1_img - *p0_img);
			p0_img = p1_img + jump_r;
			tmp -= 2*(int32)(*p0_img - *p1_img);
			p1_img = p0_img + jump_r;
			tmp += (int32)(*p1_img - *p0_img);
			/* Go on the next column */
			p0_img = p_img + jump_c;
			p1_img = p0_img + jump_r;
			tmp -= (int32)(*p1_img - *p0_img);
			p0_img = p1_img + jump_r;
			tmp += 2*(int32)(*p0_img - *p1_img);
			p1_img = p0_img + jump_r;
			tmp -= (int32)(*p1_img - *p0_img);
			break;
		case 4:
			/*         +2
			 * -1 +-----+-----+ -1
			 *    |  -  |  +  |
			 * +2 +-----4-----+ +2
			 *    |  +  |  -  |
			 * -1 +-----+-----+ -1
			           +2          */
			p1_img = p0_img + jump_r;
			tmp = (int32)(*p1_img - *p0_img);
			p0_img = p1_img + jump_r;
			tmp -= (int32)(*p0_img - *p1_img);
			/* Go on the next column */
			p0_img = p_img + jump_c;
			p1_img = p0_img + jump_r;
			tmp -= 2*(int32)(*p1_img - *p0_img);
			p0_img = p1_img + jump_r;
			tmp += 2*(int32)(*p0_img - *p1_img);
			/* Go on the next column */
			p0_img = p_img + jump_c*2;
			p1_img = p0_img + jump_r;
			tmp += (int32)(*p1_img - *p0_img);
			p0_img = p1_img + jump_r;
			tmp -= (int32)(*p0_img - *p1_img);
			break;

		case 5:
			/* -1 +-----+ +1
			 *    |  +  |
			 * +2 +-----+ -2
			 *    |  -  |
			 * -1 +-----+ +1 */
			p1_img = p0_img + jump_r;
			tmp = (int32)(*p1_img - *p0_img);
			p0_img = p1_img + jump_r;
			tmp -= (int32)(*p0_img - *p1_img);
			/* Go on the next column */
			p0_img = p_img + jump_c;
			p1_img = p0_img + jump_r;
			tmp -= (int32)(*p1_img - *p0_img);
			p0_img = p1_img + jump_r;
			tmp += (int32)(*p0_img - *p1_img);
			tmp=-1*tmp;
			break;

		case 6:
			/* +1   -2    +1
			 * +-----+-----+
			 * |  -  |  +  |
			 * +-----+-----+
			 * -1   +2    -1 */
			p1_img = p0_img + jump_r;
			tmp -= (int32)(*p1_img - *p0_img);
			/* Go on the next column */
			p0_img += jump_c;
			p1_img += jump_c;
			tmp += 2*(int32)(*p1_img - *p0_img);
			/* Go on the next column */
			p0_img += jump_c;
			p1_img += jump_c;
			tmp -= (int32)(*p1_img - *p0_img);
			tmp=-1*tmp;
			break;

		case 7:
			/*         +2
			 * -1 +-----+-----+ -1
			 *    |  +  |  -  |
			 * +2 +-----4-----+ +2
			 *    |  -  |  +  |
			 * -1 +-----+-----+ -1
			           +2          */
			p1_img = p0_img + jump_r;
			tmp = (int32)(*p1_img - *p0_img);
			p0_img = p1_img + jump_r;
			tmp -= (int32)(*p0_img - *p1_img);
			/* Go on the next column */
			p0_img = p_img + jump_c;
			p1_img = p0_img + jump_r;
			tmp -= 2*(int32)(*p1_img - *p0_img);
			p0_img = p1_img + jump_r;
			tmp += 2*(int32)(*p0_img - *p1_img);
			/* Go on the next column */
			p0_img = p_img + jump_c*2;
			p1_img = p0_img + jump_r;
			tmp += (int32)(*p1_img - *p0_img);
			p0_img = p1_img + jump_r;
			tmp -= (int32)(*p0_img - *p1_img);
			tmp=-1*tmp;
			break;
	}
	*result = tmp;

	return ret_code;
}

#endif

#ifdef __cplusplus
}
#endif

#endif /* __C4MGETFEATUREVALUE_R_H_ */
