/*******************************************************************************
 *                            C STMicroelectronics
 *    Reproduction and Communication of this document is strictly prohibited
 *      unless specifically authorized in writing by STMicroelectronics.
 *------------------------------------------------------------------------------
 *                              Imaging Division
 *------------------------------------------------------------------------------
 * $Id: facedetector_987_c.h.rca 1.2 Wed Jul 23 11:00:40 2008 vitor.schwambach@st.com Experimental $
 *------------------------------------------------------------------------------
 * $Log: facedetector_987_c.h.rca $
 * 
 *  Revision: 1.2 Wed Jul 23 11:00:40 2008 vitor.schwambach@st.com
 *  [ RAID CR #49366 ] facedetect : implement variable input image resize
 *  Determines the resize ratio based on the input image dimensions so that the actual image input to the main algorithm is ideally QVGA (if it is really QVGA or not will depend on the aspect ratio of the image).
 *  Also provides a command line option to force a change on the size of the image provided to the algorithm.
 * 
 *  Revision: 1.1 Fri Jul 18 13:56:57 2008 vitor.schwambach@st.com
 *  Initial version from AST
 ******************************************************************************/

/*******************************************************************************
 *                       Copyright (c) STMicroelectronics
 *   Reproduction and Communication of this document is strictly prohibited 
 *   unless specifically authorized in writing by STMicroelectronics
 *******************************************************************************

 *******************************************************************************

 FILENAME  : facedetector_987_c.h
 PROJECT   : Face2Face
 MODULE    : Tracking2D

 AUTHOR(S) : Claudio MARCHISIO (Advanced System Technology, Agrate)

 COPYRIGHT : STMicroelectronics, 2008

 This document contains proprietary and confidential information of the
 STMicroelectronics Group. All rights reserved.

 ----------------------------- MODULE DESCRIPTION -----------------------------

 FaceDetector_987 class declaration.

 ---------------------------------- HISTORY -----------------------------------

 2008.04.28 : (Claudio Marchisio) Creation.

 ******************************************************************************/

#ifndef __FACEDETECTOR_987_C_H_
#define __FACEDETECTOR_987_C_H_

#ifndef __FACEFINDER_HAAR_MULTI_C_H_
#include "facefinder_haar_multi_c.h"
#endif


#ifdef __cplusplus
extern "C" {
#endif


#define _MERGE_ACTIVE

typedef enum _FTResult {
	EFaceNotFound,
	EFaceFound,
	EMemoryAllocationError,
	EFrameBufferNotValid,
	ETrackingStarted,
	ETrackingNotStarted,
	ETrackingAlreadyStarted,
	EWrongArguments,
	EFrameSizeNotSupported
} FTResult;

typedef enum _TError {
	EErrNone,
	EErrInternal,
	EErrMemoryAllocation,
	EErrInitialization,
} TError;

typedef struct _FaceDetector_987
{
	uint32 iFramesBeforeRestart;
	FaceFinderHaarMulti iFaceFinderMulti;	// FaceFinder based on Haar features (multi cascades).
	uint32 iFaceScoreThreshold;
	FrameInfo iFrameInfo;				// Info about the current frame.
	AreaInfo iSearchArea;				// Info about the searching area for the color tracker.

#ifdef _MERGE_ACTIVE
//	uint32 iLastNumFaces;		// The total number of faces found during last execution.
//	Face* iLastFaces;			// The pointer to the faces found during last execution. It has iMaxPotentialFaces elements.

	uint32 iNFaces;	// The total number of faces found during last execution.
	Face* iFaces;	// The pointer to the faces found during last execution. It has iMaxPotentialFaces elements.

	c4mMatrix iFaceMatrix;
	c4mMatrix iScoreMatrix;
#endif
} FaceDetector_987;


void FaceDetector_987Ctor(FaceDetector_987* obj);
void FaceDetector_987Dtor(FaceDetector_987* obj);
void FaceDetector_987Init(FaceDetector_987* obj, const uint32 frameWidth, const uint32 frameHeight, AreaInfo* searchArea, uint32 scoreThreshold, uint32 targetFrameWidth);
void FaceDetector_987Run(FaceDetector_987* obj, const FrameInfo* frameInfo, uint32* nFaces,  Face** faces);

#if 0
protected:
	uint32 FaceDetector_987AddFace(Face* face, AreaInfo* region);
#endif


#ifdef __cplusplus
}
#endif


#endif // __FACEDETECTOR_987_C_H_
