/*******************************************************************************
 *                            C STMicroelectronics
 *    Reproduction and Communication of this document is strictly prohibited
 *      unless specifically authorized in writing by STMicroelectronics.
 *------------------------------------------------------------------------------
 *                              Imaging Division
 *------------------------------------------------------------------------------
 * $Id: facefinder_haar_multi_c.h.rca 1.2 Wed Jul 23 10:36:12 2008 vitor.schwambach@st.com Experimental $
 *------------------------------------------------------------------------------
 * $Log: facefinder_haar_multi_c.h.rca $
 * 
 *  Revision: 1.2 Wed Jul 23 10:36:12 2008 vitor.schwambach@st.com
 *  No modification.
 * 
 *  Revision: 1.1 Fri Jul 18 13:56:57 2008 vitor.schwambach@st.com
 *  Initial version from AST
 ******************************************************************************/

/*******************************************************************************
 *                       Copyright (c) STMicroelectronics
 *   Reproduction and Communication of this document is strictly prohibited 
 *   unless specifically authorized in writing by STMicroelectronics
 *******************************************************************************

 *******************************************************************************

 FILENAME  : facefinder_haar.h
 PROJECT   : Face2Face
 MODULE    : Tracking2D

 AUTHOR(S) : Claudio MARCHISIO (Advanced System Technology, Agrate)

 COPYRIGHT : STMicroelectronics, 2007

 This document contains proprietary and confidential information of the
 STMicroelectronics Group. All rights reserved.

 ----------------------------- MODULE DESCRIPTION -----------------------------

 FaceFinderHaarMulti class declaration.

 ---------------------------------- HISTORY -----------------------------------

 2007.01.23 : (Claudio Marchisio) Creation.

 ******************************************************************************/

#ifndef __FACEFINDER_HAAR_MULTI_C_H_
#define __FACEFINDER_HAAR_MULTI_C_H_


#ifndef __MATRIX_C_H_
#include "matrix_c.h"
#endif

#ifndef __INFO_C_H_
#include "info_c.h"
#endif

#ifndef __C4MCHAF_H_
#include "c4mChaf.h"
#endif


#ifdef __cplusplus
extern "C" {
#endif



typedef struct _FaceFinderHaarMultiSettings
{
	uint32 lumaWidth;			// Width of the Y plane of the frame (pixels).
	uint32 lumaHeight;			// Height of the Y plane of the frame(pixels).
	uint32 maxPotentialFaces;	// Maximum number of potential faces that can be found.

	uint32 scanScaler;			// Used to scale down the frame in order to reduce memory usage and speed up scanning.

	uint32 scanResizeStep;		// Resize step: it is a number between 0 and 2^16;
								// note that these extremes are the fixed point
								// equivalent values of the extremes 0.0f ~ 1.0f (floating point).
	uint32 scanResizeStart;		// Maximum resize value (the search starts from this value).
	uint32 scanResizeStop;		// Minimum resize value (the search ends when this value is reached).

	uint32 scanStepGridX;		// Horizontal step grid (pixels).
	uint32 scanStepGridY;		// Vertical step grid (pixels).

	uint32 scanWindowLeft;		// X coordinate of the top-left corner of the scanning area (pixels).
	uint32 scanWindowTop;		// Y coordinate of the top-left corner of the scanning area (pixels).
	uint32 scanWindowRight;		// X coordinate of the bottom-right corner of the scanning area (pixels).
	uint32 scanWindowBottom;	// Y coordinate of the bottom-right corner of the scanning area (pixels).

	uint32 scanMergeMode;		// O means get all results (rectangles).
								// 1 means calculate the mean of the resulting rectangles.
								// 2 means calculate the minimum rectangle wrapping all the resulting rectangles.
} FaceFinderHaarMultiSettings;


typedef enum _Rotation {
	ENorth = 0,
	EEast = 1,
	ESouth = 2,
	EWest = 3
} Rotation;


typedef struct _Face
{
	int32 x;
	int32 y;
	int32 width;
	int32 height;
	uint32 score;
	Rotation rotation;
} Face;


typedef enum _MergeMode {
	EDoNotMerge = 0,	// Get all results (rectangles).
	EMeanMerge = 1,		// Calculate the mean of the resulting rectangles.
	EWrapMerge = 2		// Calculate the minimum rectangle wrapping all the rectangles relative to a face.
} MergeMode;


#define SCAN_RESIZE_ROWS	1
#define SCAN_RESIZE_COLS	3
#define SCAN_STEPGRID_ROWS	1
#define SCAN_STEPGRID_COLS	2
#define SCAN_WINDOW_ROWS	1
#define SCAN_WINDOW_COLS	4
#define SCAN_MERGEMODE_ROWS	1
#define SCAN_MERGEMODE_COLS	1
#define TEMPLATE_SIZE_ROWS	1
#define TEMPLATE_SIZE_COLS	2
#define STATISTICS_ROWS		1
#define STATISTICS_COLS		3


typedef struct FaceFinderHaarMulti
{
	uint8* iBuffer;					// Memory buffer.
	c4mChafTmpType iAuxCobe;		// Auxiliary memory.
	uint32 iMaxPotentialFaces;		// Maximum number of potential faces.

	uint32 iScanScaler;				// Used to scale down the frame in order to reduce memory usage and speed up scanning.

	c4mChafSearchType iScanParam;	// Scanning parameters.

	c4mMatrixArray iStatisticsMatrixArray;	// Statistics.
	c4mMatrixArray iFaceAreasMatrixArray;	// Face areas found.
	c4mMatrixArray iFaceScoresMatrixArray;	// Face areas found.

	c4mMatrix iYMatrix;				// Y buffer relative to the current frame.

	uint32 iScanResizeData[SCAN_RESIZE_ROWS * SCAN_RESIZE_COLS];			// Resize parameters for the scanning.
	uint32 iScanStepGridData[SCAN_STEPGRID_ROWS * SCAN_STEPGRID_COLS];		// Step grid parameters for the scanning.
	uint32 iScanWindowData[SCAN_WINDOW_ROWS * SCAN_WINDOW_COLS];			// Window parameters for the scanning.
	uint32 iScanMergeModeData[SCAN_MERGEMODE_ROWS * SCAN_MERGEMODE_COLS];	// Merge mode parameters for the scanning.
	uint32 iScanSearchSchemaData[1];		// Search schema.
	uint32 iScanMaxScoreData[1];			// Max score.

//#ifdef TWO_CASCADES_THREE_ROTATIONS
//	static const uint32 iNumCascades = 2;		// Number of different cascades.
//	static const uint32 iNumTotCascades = 3;	// Number of total cascades (rotated cascades included).
//#else
//	static const uint32 iNumCascades = 1;		// Number of different cascades.
//	static const uint32 iNumTotCascades = 1;	// Number of total cascades (rotated cascades included).
//#endif
//	c4mChafSearchRotated iCascadeType[iNumCascades]; // BUTTARE
	c4mChafSearchRotated* iCascadeTypes;
	uint32 iNCascadeTypes;
	uint32 iNTotCascades;
	uint8** iRotatedFeaturex; // da cambiare in iRotatedFeatures.
	c4mChaf2SType* iCascadex; // da cambiare in iCascades.


	//c4mChaf2SType iCascades[iNumTotCascades]; // BUTTARE
	//uint8* iRotatedFeatures[iNumTotCascades]; // BUTTARE
	//Rotation iRotations[iNumTotCascades];		// Type of rotation.
	CollectData iCollectData;

	uint32* iFacesFoundArray;	// Array of found faces for each cascade.

	uint32 iNumFaces;		// The total number of faces found.
	Face* iFaces;			// The pointer to the faces found. It has iMaxPotentialFaces elements.
} FaceFinderHaarMulti;


void FaceFinderHaarMultiCtor(FaceFinderHaarMulti* obj);
void FaceFinderHaarMultiDtor(FaceFinderHaarMulti* obj);
uint32 FaceFinderHaarMultiInit(FaceFinderHaarMulti* obj, const FaceFinderHaarMultiSettings* settings);				// Initialize this object.
uint32 FaceFinderHaarMultiRun(FaceFinderHaarMulti* obj, const FrameInfo* frameInfo, 
	const AreaInfo* searchArea, 
	uint32* nFaces,  Face** faces);	// Execute the face finder.

uint32 FaceFinderHaarMultiCreateBuffers(FaceFinderHaarMulti* obj);	// Allocate memory buffers.
void FaceFinderHaarMultiDeleteBuffers(FaceFinderHaarMulti* obj);	// Release memory buffers.
void FaceFinderHaarMultiSetFaces(FaceFinderHaarMulti* obj);
void FaceFinderHaarMultiLoadFeatures(FaceFinderHaarMulti* obj);
void FaceFinderHaarMultiReleaseFeatures(FaceFinderHaarMulti* obj);



#ifdef __cplusplus
}
#endif

#endif // __FACEFINDER_HAAR_MULTI_C_H_
