/*******************************************************************************
 *                            C STMicroelectronics
 *    Reproduction and Communication of this document is strictly prohibited
 *      unless specifically authorized in writing by STMicroelectronics.
 *------------------------------------------------------------------------------
 *                              Imaging Division
 *------------------------------------------------------------------------------
 * $Id: info_c.h.rca 1.2 Wed Jul 23 10:36:12 2008 vitor.schwambach@st.com Experimental $
 *------------------------------------------------------------------------------
 * $Log: info_c.h.rca $
 * 
 *  Revision: 1.2 Wed Jul 23 10:36:12 2008 vitor.schwambach@st.com
 *  No modification.
 * 
 *  Revision: 1.1 Fri Jul 18 13:56:57 2008 vitor.schwambach@st.com
 *  Initial version from AST
 ******************************************************************************/

/*******************************************************************************
 *                       Copyright (c) STMicroelectronics
 *   Reproduction and Communication of this document is strictly prohibited 
 *   unless specifically authorized in writing by STMicroelectronics
 *******************************************************************************

 *******************************************************************************

 FILENAME  : info.h
 PROJECT   : Face2Face
 MODULE    : Tracking2D

 AUTHOR(S) : Claudio MARCHISIO (Advanced System Technology, Agrate)

 COPYRIGHT : STMicroelectronics, 2006

 This document contains proprietary and confidential information of the
 STMicroelectronics Group. All rights reserved.

 ----------------------------- MODULE DESCRIPTION -----------------------------

 Data structure declaration.

 ---------------------------------- HISTORY -----------------------------------

 2006.11.17 : (Claudio Marchisio) Creation.

 ******************************************************************************/

#ifndef __INFO_C_H_
#define __INFO_C_H_

#include "c4mTypes.h"

typedef struct _FrameInfo
{
	uint32 width;	// Width of the frame (pixels).
	uint32 height;	// Height of the frame (pixels).
	uint8* frame;	// Pointer to the buffer containing frame with YCbCr 4:2:0 planar format.
	uint32 size;	// Size of the buffer (bytes)
	uint32 time;	// Presentation time of the frame.
} FrameInfo;


// Origin of the coordinates is (0, 0).
// Pay attention to set the right and bottom fields:
// if an area has (X, Y) center and (W x H) size, than the fields are calculated as:
// centerX = X; centerY = Y; width = W; height = H;
// left = X - (W >> 1); top = Y - (H >> 1); right = left + W - 1; bottom = top + H - 1;
// New usage: the pixels field has different meaning: 
//	1) when used as input argument to pass info about a searching area: it contains
//		the minimum number of pixels that must be found by the search method; in case 
//		the search method does not find at least this number of pixels, the method itself 
//		will return 0.
//	2) when used as output argument to get info about the area found: it contains the 
//		number of significant pixels that satisfy the search method.
typedef struct _AreaInfo
{
	int32 valid;	// True if all parameters are valid, false otherwise.

	int32 centerX;	// X coordinate of the center of the area (pixels).
	int32 centerY;	// Y coordinate of the center of the area (pixels).
	uint32 width;	// Width of the area (pixels).
	uint32 height;	// Height of the area(pixels).

	int32 left;		// X coordinate of the top-left corner of the area (pixels).
	int32 top;		// Y coordinate of the top-left corner of the area (pixels).
	int32 right;	// X coordinate of the bottom-right corner of the area (pixels).
	int32 bottom;	// Y coordinate of the bottom-right corner of the area (pixels).

	uint32 pixels;	// Number of the significant pixels within the area.
} AreaInfo;


#ifdef _DEBUG_TRACKING
typedef struct _DebugInfo
{
#ifndef _USE_FXP
	double* lkTemplate;			// Pointer to the LK's template buffer.
#endif
	uint32 lkTemplateWidth;		// Width of the LK's template.
	uint32 lkTemplateHeight;	// Height of the LK's template.

	uint8* colorMask;			// Pointer to the mask buffer used by Tracker2DColor.
	uint32 colorMaskWidth;		// Width of the mask used by Tracker2DColor (pixels).
	uint32 colorMaskHeight;		// Height of the mask used by Tracker2DColor (pixels).

#ifndef _USE_FXP
	float64 colorAvgCb;			// Average Cb calculated by Tracker2DColor.
	float64 colorAvgCr;			// Average Cr calculated by Tracker2DColor.
#else
	uint32 colorAvgCb;			// Average Cb calculated by Tracker2DColor.
	uint32 colorAvgCr;			// Average Cr calculated by Tracker2DColor.
#endif

	AreaInfo colorArea;			// Info about the area containing skin.
	AreaInfo colorSearchArea;	// Info about the searching area for the color tracker.
	AreaInfo mdArea;			// Info about the moving area.
	AreaInfo mdSearchArea;		// Info about the searching area for the MD tracker.
} DebugInfo;

#endif	// _DEBUG_TRACKING



#ifdef _DEBUG_TRACKING
extern inline void ResetDebugInfo(DebugInfo* info)
{
	if (!info)
		return;

#ifndef _USE_FXP
	info->lkTemplate = 0;
#endif
	info->lkTemplateWidth = 0;
	info->lkTemplateHeight = 0;

	info->colorMask = 0;
	info->colorMaskWidth = 0;
	info->colorMaskHeight = 0;

	info->colorAvgCb = 0;
	info->colorAvgCr = 0;

	ResetAreaInfo(&info->colorArea);
	ResetAreaInfo(&info->colorSearchArea);
	ResetAreaInfo(&info->mdArea);
	ResetAreaInfo(&info->mdSearchArea);
}
#endif	// _DEBUG_TRACKING


#endif // __INFO_C_H__
