/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
* File Name          : definitions.h
* Author             : AST Robotics group
* Date First Issued  : 11 May 2007 : Version 1.0
* Description        : generic definitions and pin assignments file for Dongle.
********************************************************************************
* History:
* 28 May 2007 : Version 1.2
* 11 May 2007 : Version 1.0
*******************************************************************************
 THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS WITH
 CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
 AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
 OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
 OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
 CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

/* Define to prevent recursive inclusion ------------------------------------ */
#ifndef __DEFINITIONS_H
#define __DEFINITIONS_H

// *-------------*-------------*------------*
// | UART2_RxD   | N.A.        | UART2_RxD  |
// | UART2_TxD   | N.A.        | UART2_TxD  |
// | I2C_SCL     | P3.4        | SW_I2C_SCL |
// | I2C_SDA     | P3.5        | SW_I2C_SDA |
// | CAM_CE      | P3.6        | GPIO_OUT   |
// | CAM_PCLK    | P3.0        | Ext_Dma_In |
// | CAM_VSYNC   | P3.2        | EXT_INT_2  |
// | CAM_CLK_IN  | P3.7        | T1_PWM_OUT |
// | CAM_BUS     | P9.ALL      | GPIO_IN    |
// | LED_R       | P6.0        | GPIO_OUT   |
// | SW_1234     | P7.0123     | GPIO_IN    |
// | USB_Dp      | P7.5        | GPIO_OUT   |
// *-------------*-------------*------------*

//#define DemoUART
//#define x24_UartDebug
#define NoUart
#define NoUSB
#define NoSync

#define Cam724

//IRQ Priority    // 0 Highest, 15 Lowest
#define SERVO_TIM_ITPriority      0
#define x24_DMA_ITPriority        1
#define x24_VSYNC_INT_ITPriority  2
#define x24_SYNC_TIM_IPriority    3
#define USB_Priority              4

// BOARD CONFIGURATION
#define SERVO_GPIO            GPIO2
#define SERVO_GPIO_Pin        (GPIO_Pin_3 + GPIO_Pin_4)
#define SERVO_TIM             TIM0
#define SERVO_TIM_ITLine      TIM0_ITLine
#define SERVO_TIM_Prescaler   192  // 192 -> Tt=1/48MHz*192=4us
#define SERVO_TIM_Full_Period 5000 // 5000 -> Tp=1/48MHz*5000=20ms
#define SERVO_OffsetStep      125  // 125 -> To=(125*4us)=0.5ms
#define SERVO_MaxStep         500  // 500 -> Tm=((500+125)*4us)=2.5ms

#define LEDR_GPIO GPIO6
#define LEDR_GPIOx_Pin GPIO_Pin_1

#ifndef NoUart
  #define UartNum UART2
  #define __UartNum __UART2
  #define UART_TX_GPIO GPIO3
  #define UART_TX_GPIOx_Pin GPIO_Pin_5
  #define UART_TX_GPIOx_OutAlt GPIO_OutputAlt3
  #define UART_RX_GPIO GPIO3
  #define UART_RX_GPIOx_Pin GPIO_Pin_1
#endif

#define USB_Dp_PullUp_GPIO GPIO7
#define USB_Dp_PullUp_GPIOx_Pin GPIO_Pin_5

#define SW1_GPIO GPIO7
#define SW1_GPIOx_Pin GPIO_Pin_0
#define SW2_GPIO GPIO7
#define SW2_GPIOx_Pin GPIO_Pin_1
#define SW3_GPIO GPIO7
#define SW3_GPIOx_Pin GPIO_Pin_2
#define SW4_GPIO GPIO7
#define SW4_GPIOx_Pin GPIO_Pin_3

// SW_I2C CONFIGURATION
#define SW_I2C_SCL_GPIO GPIO3
#define SW_I2C_SCL_GPIOx_Pin GPIO_Pin_4
#define SW_I2C_SDA_GPIO GPIO3
#define SW_I2C_SDA_GPIOx_Pin GPIO_Pin_5

// x24 CONFIGURATION
#define x24_CE_GPIO GPIO3
#define x24_CE_GPIOx_Pin GPIO_Pin_6

#define x24_PCLK_GPIO GPIO3 // always GPIO3 (P3.0/P3.1)
#define x24_PCLK_GPIOx_Pin GPIO_Pin_0 // P3.0/P3.1
#define x24_PCLK_DMA_Channel DMA_Channel0
#define x24_PCLK_DMA_Channel_N 0
#define x24_PCLK_EXT_DMA_REQ_x DMA_SRC_External_Req0

#define x24_VSYNC_GPIO GPIO3
#define x24_VSYNC_GPIOx_Pin GPIO_Pin_2
#define x24_VSYNC_INT_WIU_Line WIU_Line2
#define x24_VSYNC_INT_WIU_Line_N 2
#define x24_VSYNC_INT_ITx_LINE EXTIT0_ITLine

#ifndef NoSync
  #define x24_SYNC_LED_GPIO GPIO6
  #define x24_SYNC_LED_GPIOx_Pin GPIO_Pin_2
  #define x24_SYNC_IN_GPIO GPIO4
  #define x24_SYNC_IN_GPIOx_Pin GPIO_Pin_4
  #define x24_SYNC_IN_ALT GPIO_InputAlt1
  #define x24_VSYNC_T_GPIO GPIO4
  #define x24_VSYNC_T_GPIOx_Pin GPIO_Pin_5
  #define x24_VSYNC_T_ALT GPIO_InputAlt1
  #define x24_SYNC_TIM TIM2
  #define x24_SYNC_TIM_ITx_LINE TIM2_ITLine
  #define x24_SYNC_TIM_Res_us 1000
  #define x24_SYNC_OUT_GPIO GPIO4
  #define x24_SYNC_OUT_GPIOx_Pin GPIO_Pin_6
  #define x24_SYNC_OUT_ALT GPIO_OutputAlt2
  #define x24_SYNC_OUT_TIM TIM3
  #define x24_SYNC_OUT_FPS 30
#endif

#define x24_EXT_CLK_GPIO GPIO3
#define x24_EXT_CLK_GPIOx_Pin GPIO_Pin_7
#define x24_EXT_CLK_GPIO_Alt GPIO_OutputAlt3
#define x24_EXT_CLK_TIMx TIM1

#define x24_DATA_BUS_GPIO GPIO9

// x24 I2C configuration
#define x24_I2C_HwConfig SWI2C_HwConfig
#define x24_I2CStart SWI2CStart
#define x24_I2CStop SWI2CStop
#define x24_I2CWriteByte SWI2CWriteByte
#define x24_I2CReadByte SWI2CReadByte
#define x24_I2CReadACK SWI2CReadACK
#define x24_I2CSendACK SWI2CSendACK


#define MY_GPIO GPIO2
#define MY_GPIOx_Pin GPIO_Pin_2
#define MY MY_GPIO->DR[MY_GPIOx_Pin<<2]
#define MY_ON() MY=0xFF
#define MY_OFF() MY=0x00

// other definitions
#define SERVO_PORT SERVO_GPIO->DR[SERVO_GPIO_Pin<<2]
#define LEDR LEDR_GPIO->DR[LEDR_GPIOx_Pin<<2]
#define LED_SYNC x24_SYNC_LED_GPIO->DR[x24_SYNC_LED_GPIOx_Pin<<2]
#define USB_Dp_PullUp USB_Dp_PullUp_GPIO->DR[USB_Dp_PullUp_GPIOx_Pin<<2]
#define SW1 SW1_GPIO->DR[SW1_GPIOx_Pin<<2]
#define SW2 SW2_GPIO->DR[SW2_GPIOx_Pin<<2]
#define SW3 SW3_GPIO->DR[SW3_GPIOx_Pin<<2]
#define SW4 SW4_GPIO->DR[SW4_GPIOx_Pin<<2]

#define LED_ON(Led) Led=0xFF;
#define LED_OFF(Led) Led=0x00;
#define IS_BTN_PRESSED(Btn) (Btn==0)
#define WAIT_BTN(Btn) { while(!IS_BTN_PRESSED(Btn)); while(IS_BTN_PRESSED(Btn));}
#define USB_Dp_PullUp_OFF(); USB_Dp_PullUp=0xFF;
#define USB_Dp_PullUp_ON(); USB_Dp_PullUp=0x00;

#endif

/******************* (C) COPYRIGHT 2007 STMicroelectronics *****END OF FILE****/
