/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
* File Name          : Demo.c
* Author             : AST Robotics group
* Date First Issued  : 11 June 2007 : Version 1.0
* Description        :
********************************************************************************
* History:
* 11 June 2007 : Version 1.0
*******************************************************************************
 THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS WITH
 CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
 AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
 OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
 OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
 CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/


/* Standard include ----------------------------------------------------------*/
#include "Demo.h"
#include "DemoBlobSearch.h"
#include "DemoHarris.h"
#include "DemoFast.h"
#include "91x_map.h"
#include "utils.h"
#include "definitions.h"
#include "definitionsLib.h"
#include "CamInt_x24.h"
#include "usb_lib.h"
#include "usb_conf.h"
#include "usb_prop.h"
#include "usb_pwr.h"
#include "usb_config.h"
#include "serial_protocol.h"
#include "RGBCube.h"
#include "blobsearch.h"

/* Include of other module interface headers ---------------------------------*/
/* Local includes ------------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define TimeDiv 250000000
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
const u16 SyncOutPeriods[150] ={9999, 8332, 7142, 6249, 5555, 4999, 4545, 4166, 3846, 3571, 3333, 3125, 2941, 2777, 2631, 2500, 2381, 2272, 2174, 2083, 2000, 1923, 1852, 1786, 1724, 1666, 1613, 1562, 1515, 1470, 1428, 1389, 1351, 1316, 1282, 1250, 1219, 1190, 1163, 1136, 1111, 1087, 1064, 1042, 1020, 1000, 980, 961, 943, 926, 909, 893, 877, 862, 847, 833, 820, 806, 794, 781, 769, 757, 746, 735, 725, 714, 704, 694, 685, 676, 667, 658, 649, 641, 633, 625, 617, 610, 602, 595, 588, 581, 575, 568, 562, 555, 549, 543, 538, 532, 526, 521, 515, 510, 505, 500, 495, 490, 485, 481, 476, 472, 467, 463, 459, 454, 450, 446, 442, 439, 435, 431, 427, 424, 420, 417, 413, 410, 406, 403, 400, 397, 394, 391, 388, 385, 382, 379, 376, 373, 370, 368, 365, 362, 360, 357, 355, 352, 350, 347, 345, 342, 340, 338, 336, 333};
vu8 SenderInterface = 0;

u8 *pImgBufferData;
u32 BufferSize;
u8 USB_RxBuffer[USB_RxFifoDim];
u32 USB_RxBufferLen=0;
const u8 PresentationString[] = PresentationStringConst;
Tx24_ImgJob Job[2], *CompletedJob=0;
u8 CurrentJob;
u8 CamState=0; // 0:Uniitialized, 1:Running, 2:Paused
u8 CamStreaming=0; // 1:Streaming an image
u16 LastFC = 0;
u8 ActiveDemo = DEMO_Undefined;
TTimeStamp Tcam1, Tcam2;
vs32 OldErr=0;
vs32 Ierr=0, Nerr=0;

/* Private function prototypes -----------------------------------------------*/
int USB_ReceivedMSG(TMsg *Msg);
void USB_SendMsg(TMsg *Msg);
void UART_SendMsg(TMsg *Msg);
int UART_ReceivedMSG(TMsg *Msg);
void CameraSetParam(u8 ImgFormat, u16 Width, u16 Height, u8 Framerate,
                    u8 SensorMode, u8 HalfSysClock, u8 JpegCompr, u8 JpegDerat);
void GetCompletedJob(void);
int Handle_CMD_CameraGetImage(TMsg *Msg);
/*inline*/ void BUILD_REPLY_HEADER(TMsg *Msg);
int Binarize(u8 *Img, u16 Width, u16 Height, u8 Threshold);

/* Private functions ---------------------------------------------------------*/

void Led_SM(void)
{
  static int Count=0;

  Count++;
  if (Count<50000) {
    LED_ON(LEDR);
  } else {
    LED_OFF(LEDR);
    if (Count>60000) {
      Count=0;
    }
  }
}

/*inline*/ void BUILD_REPLY_HEADER(TMsg *Msg)
{
  Msg->Data[0]=Msg->Data[1];
  Msg->Data[1]=DEV_ADDR;
  Msg->Data[2]+=CMD_Reply_Add;
}

void GetCompletedJob(void)
{
  if (Job[CurrentJob].State==x24_StateAcquisitionEnd) {
    CompletedJob=&Job[CurrentJob];
    CurrentJob^=1;
//    x24_WriteReg8(0x03ba,0x01);
//    	  x24_WriteReg8(0x1380,0);  //wb
//    	  x24_WriteReg8(0x1080,2);  //ae
//    	  x24_WriteReg16(0x1095,1000);
//    	  x24_WriteReg16(0x109d,240);
//    	  x24_WriteRegF900(0x10a1,1.0);

  }
}

int CamJob_Pause(void)
{
  if (CamState==0) return 0;// Camera OFF
  // if (CamState==2) return 1; // already paused
  if (CamState==1) {
    VIC_ITCmd(x24_VSYNC_INT_ITx_LINE, DISABLE);
    GetCompletedJob();
    Job[CurrentJob].pNext=0;
    CamState=2;
    VIC_ITCmd(x24_VSYNC_INT_ITx_LINE, ENABLE);
  }
  return 1;
}

void CamJob_SM(void)
{
  switch (CamState) {
  case 0: // Camera OFF
    break;
  case 1: // cam running


    GetCompletedJob();
    break;
  case 2: // cam paused
    if (!CamStreaming) {// && (!BlobSearching)) {
      VIC_ITCmd(x24_VSYNC_INT_ITx_LINE, DISABLE);
      if (Job[CurrentJob].State==x24_StateAcquisitionEnd) {
        CompletedJob=&Job[CurrentJob];
        CurrentJob^=1;
        Job[0].pNext = &Job[1];
        Job[1].pNext = &Job[0];
        x24_GrabFrameStart(&Job[CurrentJob]);
      } else {
        Job[CurrentJob].pNext = &Job[CurrentJob^1];
        Job[CurrentJob^1].pNext = &Job[CurrentJob];
      }
      VIC_ITCmd(x24_VSYNC_INT_ITx_LINE, ENABLE);
      CamState=1;

    }
    break;
  }
}

int HandleMSG(TMsg *Msg)
//  DestAddr | SouceAddr | CMD | PAYLOAD
//      1          1        1       N
{
  const u8 *p1;
  u8 *p2;
  u32 i;
  u8 App8;

  if (Msg->Len<2) return 0;
  if (Msg->Data[0]!=DEV_ADDR) return 0;
  switch (Msg->Data[2]) { // CMD
	  case CMD_PING:
		  if (Msg->Len!=3) return 0;
		  BUILD_REPLY_HEADER(Msg);
		  Msg->Len=3;
		  return 1;
	  case CMD_Read_PresString:
		  if (Msg->Len!=3) return 0;
		  BUILD_REPLY_HEADER(Msg);
		  p1=PresentationString;
		  p2=&Msg->Data[3];
		  i=0;
		  while (*p1) {
			  *p2=*p1;
			  p1++;
			  p2++;
			  i++;
		  }
		  Msg->Len=3+i;
		  return 1;
//  }
//  if(ActiveDemo == DEMO_Undefined)
  //switch (Msg->Data[2]) { // CMD
	  case CMD_SelectDemo:
		  if (Msg->Len!=4) return 0;
		  BUILD_REPLY_HEADER(Msg);
		  Msg->Len=3;
		  switch(Msg->Data[3]) {
		  case DEMO_BlobSearch:
		  case DEMO_Jpeg:
			  DemoBlobSearch(Msg);
			  break;
		  case DEMO_FaceDetection:
			  //			DemoFaceDetection();
			  break;
		  case DEMO_Harris:
			  DemoHarris(Msg);
			  break;
		  case DEMO_Fast:
			  DemoFast(Msg);
			  break;
		  }

		  return 1;
	  case CMD_Reset:
		  x24_DisableCamera();
		  break;
		  case CMD_CameraGetState: // 0:Uniitialized, 1:Running, 2:Paused
			  if (Msg->Len!=3) return 0;
			  BUILD_REPLY_HEADER(Msg);
			  Msg->Data[3]=CamState;
			  Msg->Len=4;
			  return 1;
		  case CMD_CameraStart: // ImgFormat_U8, Width_U16, Height_U16, Framerate_U8, SensorMode_U8, HalfSysClock_U8, JpegCompr_U8, JpegDerat_U8
			  if (Msg->Len!=13) return 0;
			  if (CamState>0) return 0; // camera already initialized
			  BUILD_REPLY_HEADER(Msg);
			  CameraSetParam(Msg->Data[3], Deserialize(&Msg->Data[4], 2), Deserialize(&Msg->Data[6], 2),
					  Msg->Data[8], Msg->Data[9], Msg->Data[10], Msg->Data[11], Msg->Data[12]);

			  CamState=1;
			  Msg->Len=3;
			  return 1;
			  //  case CMD_CameraStop:
			  //    if (Msg->Len!=3) return 0;
			  //    BUILD_REPLY_HEADER(Msg);
			  //    x24_HwStop();
			  //    Msg->Len=3;
			  //    return 1;
		  case CMD_CameraGetImage: // FrameNumber_U16
			  if (Msg->Len!=5) return 0;
			  BUILD_REPLY_HEADER(Msg);
			  if(!Handle_CMD_CameraGetImage(Msg)) return 0;
			  return 1;
		  case CMD_CameraContinue:
			  if (Msg->Len!=3) return 0;
			  BUILD_REPLY_HEADER(Msg);
			  CamStreaming=0;
			  Msg->Len=3;
			  return 1;
		  case CMD_RGBCubeClear:
			  if (Msg->Len!=3) return 0;
			  BUILD_REPLY_HEADER(Msg);
			  RGBCube_Clear();
			  //    ClearRGBCube();
			  Msg->Len=3;
			  return 1;
		  case CMD_RGBCubeSetClassR: // RG_U8, RB_U8, ClassID_U8
			  if (Msg->Len!=6) return 0;
			  BUILD_REPLY_HEADER(Msg);
			 RGBCube_AddRed(Msg->Data[3], Msg->Data[4], Msg->Data[5]);
			  //    InitRGBCubeRed(Msg->Data[3], Msg->Data[4], Msg->Data[5]);
			  Msg->Len=3;
			  return 1;
		  case CMD_RGBCubeSetClassG: // GR_U8, GB_U8, ClassID_U8
			  if (Msg->Len!=6) return 0;
			  BUILD_REPLY_HEADER(Msg);
			  RGBCube_AddGreen(Msg->Data[3], Msg->Data[4], Msg->Data[5]);
			  //    InitRGBCubeGreen(Msg->Data[3], Msg->Data[4], Msg->Data[5]);
			  Msg->Len=3;
			  return 1;
		  case CMD_RGBCubeSetClassB: // BR_U8, BG_U8, ClassID_U8
			  if (Msg->Len!=6) return 0;
			  BUILD_REPLY_HEADER(Msg);
			  RGBCube_AddBlue(Msg->Data[3], Msg->Data[4], Msg->Data[5]);
			  //    InitRGBCubeBlue(Msg->Data[3], Msg->Data[4], Msg->Data[5]);
			  Msg->Len=3;
			  return 1;
		  case CMD_RGBCubeSetClassCube: // Rmin_U8, Rmax_U8, Gmin_U8, Gmax_U8, Bmin_U8, Bmax_U8, ClassID_U8
			  if (Msg->Len!=11) return 0;
			  BUILD_REPLY_HEADER(Msg);
			  if(Msg->Data[3] == 1) {
				  RGBCube_AddCube(Msg->Data[4], Msg->Data[5], Msg->Data[6], Msg->Data[7], Msg->Data[8], Msg->Data[9], Msg->Data[10]);
			  } else {
				  RGBCube_SubCube(Msg->Data[4], Msg->Data[5], Msg->Data[6], Msg->Data[7], Msg->Data[8], Msg->Data[9], Msg->Data[10]);
			  }
			  Msg->Len=3;
			  return 1;
		  case CMD_BlobSearchSetOptions: // GridSize_U8, SerchRect(Top_U8, Bottom_U8, Left_U8, Right_U8)
			  if (Msg->Len!=8) return 0;
			  BUILD_REPLY_HEADER(Msg);
			  SearchOptions.GridSize=Msg->Data[3];
			  SearchOptions.SearchRect.Top=Msg->Data[4];
			  SearchOptions.SearchRect.Bottom=Msg->Data[5];
			  SearchOptions.SearchRect.Left=Msg->Data[6];
			  SearchOptions.SearchRect.Right=Msg->Data[7];
			  Msg->Len=3;
			  return 1;
		  case CMD_BlobSearchSetClassParam:// MinPer_U16,  MaxPer_U16, MinArea_U16, MaxArea_U16, MinCirc_U8, MaxCirc_U8, ClassID_U8
			  if (Msg->Len!=14) return 0;
			  BUILD_REPLY_HEADER(Msg);
			  App8=Msg->Data[13]-1;
			  if (App8>=MAX_COLOR_SETS) return 0;
			  ColorSetParam[App8].MinPer=Deserialize(&Msg->Data[3], 2);
			  ColorSetParam[App8].MaxPer=Deserialize(&Msg->Data[5], 2);
			  ColorSetParam[App8].MinArea=Deserialize(&Msg->Data[7], 2);
			  ColorSetParam[App8].MaxArea=Deserialize(&Msg->Data[9], 2);
			  ColorSetParam[App8].MinCirc=Msg->Data[11];
			  ColorSetParam[App8].MaxCirc=Msg->Data[12];
			  Msg->Len=3;
			  return 1;
		  case CMD_BlobSearchGetBlobs:
			  if (Msg->Len!=3) return 0;
			  BUILD_REPLY_HEADER(Msg);
			  if(!Handle_CMD_BlobSearchGetBlobs(Msg)) return 0;
			  return 1;
		  case CMD_BlobSearchFmiSaveBS:
			  if (Msg->Len!=3) return 0;
			  BUILD_REPLY_HEADER(Msg);
			  if(!Handle_CMD_FmiSaveBs(Msg)) return 0;
			  return 1;
		  case CMD_BlobSearchFmiLoadBS:
			  if (Msg->Len!=3) return 0;
			  BUILD_REPLY_HEADER(Msg);
			  if(!Handle_CMD_FmiLoadBs(Msg)) return 0;
			  return 1;
		  case CMD_HarrisGetPoints:
			  if (Msg->Len!=4) return 0;
			  BUILD_REPLY_HEADER(Msg);
			  if(!Handle_CMD_GetHarrisPoints(Msg)) return 0;
			  return 1;
		  case CMD_HarrisSettings:
			  if (Msg->Len!=3+6) return 0;
			  BUILD_REPLY_HEADER(Msg);
			  //if(!Handle_CMD_BlobSearchGetBlobs(Msg)) return 0;
			  if(!Handle_CMD_HarrisSettings(Msg)) return 0;
			  return 1;
		  case CMD_FastGetPoints:
			  if (Msg->Len!=4) return 0;
			  BUILD_REPLY_HEADER(Msg);
			  if(!Handle_CMD_GetFastPoints(Msg)) return 0;
			  return 1;
		  case CMD_FastSettings:
			  if (Msg->Len!=3+4) return 0;
			  BUILD_REPLY_HEADER(Msg);
			  if(!Handle_CMD_FastSettings(Msg)) return 0;
			  return 1;
		  default:
			  return 0;
  }
}

void CameraSetParam(u8 ImgFormat, u16 Width, u16 Height, u8 Framerate,
                    u8 SensorMode, u8 HalfSysClock, u8 JpegCompr, u8 JpegDerat)
{
  Tx24_InitTypeDef x24_InitStruct;

  /* Initialize the x24 module */
  x24_StructInit(&x24_InitStruct);
  x24_InitStruct.SensorMode=SensorMode;
  x24_InitStruct.DesiredFrameRate_Num=Framerate*250;
  x24_InitStruct.DesiredFrameRate_Den=250;
  x24_InitStruct.ExtClockFreqMhz_Num=8;
  x24_InitStruct.ExtClockFreqMhz_Den=1;
  x24_InitStruct.JPEGClockDerate=JpegDerat;
  x24_InitStruct.SyncEnabled=0;

  x24_InitStruct.ImageWidth=Width;
  x24_InitStruct.ImageHeight=Height;
  x24_InitStruct.ImageFormat=ImgFormat; // x24_ImageFormat_RGB_565 x24_ImageFormat_JPG
  x24_InitStruct.JPEGSqueezeValue=JpegCompr; // use 100 [800x600], 150 [1024x768], 220 [1600x1200]
  x24_InitStruct.SysClock=HalfSysClock*2.0; // 35MHz; Range: [35:270] MHz  320*240@10Hz RGB_565

  x24_InitStruct.MemBlocksNum = 2;
  x24_InitStruct.ImgBlocksNum = 1;
  x24_InitStruct.ImgBlockSize = BufferSize;
  x24_InitStruct.pDataMemory = pImgBufferData;
//  x24_InitStruct.ExposureCompensation = 11;


//  SetExtClockRegisters(8, 1);


  x24_Init(&x24_InitStruct);

  if(ActiveDemo == DEMO_Harris) {
	  HarrisInitalization();
  }
//  StartTimSync();

//  LED_ON(LEDR);
//  MSDelay(10000);
//  LED_OFF(LEDR);



  Job[0].pNext = &Job[1];
  Job[1].pNext = &Job[0];
  CurrentJob=0;
  x24_GrabFrameStart(&Job[0]);


}

int Binarize(u8 *Img, u16 Width, u16 Height, u8 Threshold)
{
	int i, j;
	for(i=0;i<Height;i++) {
		for(j=0;j<Width;j++) {
			Img[j+i*Width] = Img[j+i*Width]>Threshold ? 0xFF : 0;
		}
	}
}

int Handle_CMD_CameraGetImage(TMsg *Msg)
{
  u8 *pS, *pD;
  u16 Row;
  int i;
  u16 DataLen;
  u16 N = 1;

  x24_WriteReg8(0x0280,0);

  DataLen = x24_Engine.Row_Size;
  while((DataLen+16)>TMsg_MaxLen){
    DataLen=DataLen/2;
    N*=2;
  }
  Row=Deserialize(&Msg->Data[3], 2);

  if (Row==0) {// first row
    if (!CamStreaming) {
      if (!CamJob_Pause()) return 0;
    }

    if (x24_Engine.Res_Colour_Type==x24_ImageFormat_YCbCr_400) {
    	YCbCrBug((u8 *)CompletedJob->pFirstBlock->pData, x24_Engine.Row_Size, (u16)CompletedJob->pFirstBlock->CurrentBlockRow);
    }

   // Binarize((u8 *)CompletedJob->pFirstBlock->pData, x24_Engine.Row_Size, (u16)CompletedJob->pFirstBlock->CurrentBlockRow, 120);

    CamStreaming=1;
    Serialize(&Msg->Data[7], x24_Engine.Res_Width, 2);
    Serialize(&Msg->Data[9], x24_Engine.Res_Height, 2);
    Serialize(&Msg->Data[11], DataLen, 2);
    Serialize(&Msg->Data[13], CompletedJob->CurrImgRow*N, 2);
    Msg->Data[15] = x24_Engine.Res_Colour_Type;
    pD=&Msg->Data[16];
    Msg->Len=DataLen+16;
  } else {
    if (!CamStreaming) return 0;
    pD=&Msg->Data[7];
    Msg->Len=DataLen+7;
  }
  Serialize(&Msg->Data[3], CompletedJob->FrameCount, 2);
  Serialize(&Msg->Data[5], Row, 2);
  pS=(u8*)CompletedJob->pFirstBlock->pData + Row*DataLen;
  for (i=0; i<DataLen; i++) {
    *pD=*pS;
    pD++;
    pS++;
  }
  return 1;
}


void USB_SendMsg(TMsg *Msg)
{
  int CountMsg, CountOut;
  static u8 My_Buffer[60];

  CHK_ComputeAndAdd(Msg);
  CountMsg=0;
  CountOut=0;
  while (CountMsg<Msg->Len) {
    CountOut+=ByteStuffCopyByte(&My_Buffer[CountOut], Msg->Data[CountMsg]);
    CountMsg++;
    if (CountOut>=59) {
      USB_WriteBuffer_1(My_Buffer, CountOut);
      CountOut=0;
    }
  }
  My_Buffer[CountOut]=TMsg_EOF;
  CountOut++;
  USB_WriteBuffer_1(My_Buffer, CountOut);
}

int USB_ReceivedMSG(TMsg *Msg)
{
  u8 Data;

  while (CharFifo_GetFull(&USB_RxFifo)) {
//    VIC_ITCmd(USBLP_ITLine, DISABLE);
    Data=CharFifo_Extract(&USB_RxFifo);
//    VIC_ITCmd(USBLP_ITLine, ENABLE);
    USB_RxBuffer[USB_RxBufferLen]=Data;
    USB_RxBufferLen++;
    if (Data==TMsg_EOF) {
      USB_RxBufferLen=0;
      if (ReverseByteStuffCopy(Msg, USB_RxBuffer)) {
        if (CHK_CheckAndRemove(Msg)) { // check message integrity
          return 1;
        }
      }
      return 0;
    }
    if (USB_RxBufferLen>=USB_RxFifoDim) { // Error: frame too long
      USB_RxBufferLen=0;
      return 0;
    }
  }
  return 0;
}

int UART_ReceivedMSG(TMsg *Msg)

{

	u16 i, j, k, j2;

	u16 DmaCounter, length;

	u8  Data;

	u16 source=0;

	u8 inc = 0;



	if( DMA_GetChannelStatus(Channel1) ){

		DmaCounter = (UART_RxBufferSize - (DMA_Channel1->CC & 0xFFF) ); // DMA_TrsfSisze_Mask

		if( DmaCounter >= Uart_Engine.StartOfMsg ){

			length = DmaCounter - Uart_Engine.StartOfMsg;

		} else {

			length = UART_RxBufferSize + DmaCounter - Uart_Engine.StartOfMsg;

		}

		j=Uart_Engine.StartOfMsg;

		for(k=0; k<length; k++){

			Data=UART_RxBuffer[j];

			j++;

			if (j>=UART_RxBufferSize) {

				j=0;

			}

			if (Data==TMsg_EOF) {

				j=Uart_Engine.StartOfMsg;

				for (i=0; i<k; i+=inc) {

//					            inc = ReverseByteStuffCopyByte(&UART_RxBuffer[j],&Msg->Data[source]);

					j2=(j+1)%UART_RxBufferSize;

					inc = ReverseByteStuffCopyByte2(UART_RxBuffer[j], UART_RxBuffer[j2], &Msg->Data[source]);

					if(inc == 0){

						Uart_Engine.StartOfMsg = j2;

						return 0;

					}

					j = (j + inc) % UART_RxBufferSize;

					source++;

				}

				Msg->Len=source;

				j = (j + 1) % UART_RxBufferSize; // skip TMsg_EOF

				Uart_Engine.StartOfMsg = j;

				if (CHK_CheckAndRemove(Msg)) { // check message integrity

					return 1;

				}

			}

		}

		if( length > Uart_Msg_Max_Size ) {

			Uart_Engine.StartOfMsg = DmaCounter;

		}

	}

	return 0;

}
void UART_SendMsg(TMsg *Msg)
{
  int CountMsg, CountOut;
  u8 ciccio32[2];

  CHK_ComputeAndAdd(Msg);
  CountMsg=0;

  DE_ON(DE);

  MSDelay(1);
  while (CountMsg<Msg->Len) {
    CountOut=ByteStuffCopyByte(ciccio32, Msg->Data[CountMsg]);
    CountMsg++;
    UART_SendBuffer(UART0, ciccio32, CountOut);
  }

  ciccio32[0]=TMsg_EOF;
  UART_SendBuffer(UART0, ciccio32, 1);

  while(UART_GetFlagStatus(UART0,UART_FLAG_Busy));
  DE_OFF(DE);
}


void YCbCrBug(u8 *Img, u16 Width, u16 Height)
{
  u32 i;
  u8 app, *p1;

  if(LastFC == CompletedJob->FrameCount) return;

  LastFC = CompletedJob->FrameCount;

  for (i=0; i<(Width*Height/2); i++) {
    p1=Img+1;
    app=*p1;
    *p1=*Img;
    *Img=app;
    Img+=2;
  }
}

void x24_TIM_SYNC_Handler(void)
{
#ifndef NoSync
  const u32 Kp=32, Ki=3;
  const s32 L1=254, L2=258;
  s32 DerErr, err, T1, T2, RefDelta;
  u32 app;
  static u32 TimeGlobal=0;
  u32 Status;
  u16 App16, myApp;

  Status=x24_SYNC_TIM->SR;
  if (Status & TIM_FLAG_TO) {
    TimeGlobal+=x24_SYNC_TIM_Res_us;
    msTimeGlobal+=1;
//    TIM_ClearFlag(x24_SYNC_TIM, TIM_FLAG_TO);
    x24_SYNC_TIM->SR &= ~TIM_FLAG_TO;
  }
  if (Status & TIM_FLAG_IC1) {    // Camera VSync (Rising)
    App16=x24_SYNC_TIM->IC1R+4;
    Tcam1.OldGlobal=Tcam1.CapGlobal;
    Tcam1.CapGlobal=TimeGlobal+App16;
    if ( (Status & TIM_FLAG_TO) && (App16>(x24_SYNC_TIM_Res_us/2)) ) {
      Tcam1.CapGlobal-=x24_SYNC_TIM_Res_us;
    }
    Tcam1.CapPartial=Tcam1.CapGlobal-Tcam1.OldGlobal;
//    TIM_ClearFlag(x24_SYNC_TIM, TIM_FLAG_IC1);
    x24_SYNC_TIM->SR &= ~TIM_FLAG_IC1;
  }
  if (Status & TIM_FLAG_IC2) {    // Sync CLK IN (Rising)
    App16=x24_SYNC_TIM->IC2R+4;
    Tcam2.OldGlobal=Tcam2.CapGlobal;
    Tcam2.CapGlobal=TimeGlobal+App16;
    if ( (Status & TIM_FLAG_TO) && (App16>(x24_SYNC_TIM_Res_us/2)) ) {
      Tcam2.CapGlobal-=x24_SYNC_TIM_Res_us;
    }
    Tcam2.CapPartial=Tcam2.CapGlobal-Tcam2.OldGlobal;
//    TIM_ClearFlag(x24_SYNC_TIM, TIM_FLAG_IC2);
    x24_SYNC_TIM->SR &= ~TIM_FLAG_IC2;

    if (x24_Engine.SyncEnabled) { // control loop
//      LED_OFF(LED_SYNC)
      RESET_FLAG(x24_FLAG_SYNC);
      RefDelta=Tcam2.CapPartial;
      T1=Tcam1.CapGlobal;
      T2=Tcam2.CapGlobal;
      err=T2-T1;
      if (err>(RefDelta/2)) {
        err-=RefDelta;
      } else {
        if (err<(-RefDelta/2)) {
          err+=RefDelta;
        }
      }
      DerErr=err-OldErr;
      OldErr=err;
      Ierr+=err*Ki;
      Ierr=SaturaLH(Ierr, (RefDelta*L1), (RefDelta*L2));
      app=Ierr + err*Kp + DerErr*0;
      app=SaturaLH(app, (RefDelta*L1), (RefDelta*L2));
      app/=256;
      DesiredFrameRate=SaturaLH(TimeDiv/app, 250, 30*250); // 1Hz, 30Hz


//      x24_WriteReg8(x24_bDesiredFrameRate_Den, 250); // DEN
//      x24_WriteReg16(x24_uwDesiredFrameRate_Num, DesiredFrameRate); // NUM
  //    VIC_SWITCmd(x24_SYNC_SW_IT_LINE, ENABLE);
//      VIC0->SWINTR |= (0x01 << x24_SYNC_SW_IT_LINE);

      //      x24_WriteReg16_Wait(x24_uwDesiredFrameRate_Num, DesiredFrameRate, 1);
//      DesiredFrameRate=5*250;

      myApp = DesiredFrameRate<<8;
      DesiredFrameRate=(DesiredFrameRate>>8)+myApp;
//      i2c_write_reg(x24_uwDesiredFrameRate_Num, (u8*)&DesiredFrameRate, 2);
      x24_WriteReg16(x24_uwDesiredFrameRate_Num, DesiredFrameRate);

      if (Abs(err)<5000) {
        if (Nerr<10) {
          Nerr++;
        }
      } else {
        Nerr=0;
      }
      if (Nerr<10) {
  //        LED_OFF(LED_SYNC);
      } else {
//        LED_ON(LED_SYNC);
        SET_FLAG(x24_FLAG_SYNC);
//        if(x24_Engine.SyncEnabled==1) { // Master Sync
//          x24_Engine.SendSyncFC = 1;
//        }
        if(x24_Engine.SyncEnabled==2) { // Slave Sync
          if (x24_Engine.MasterFrameCount) {
            x24_Engine.FrameCount = x24_Engine.MasterFrameCount;
            x24_Engine.MasterFrameCount=0;
          }
        }
      }
    }
  }

#endif
}

void StartSyncOut(u8 FramerateTime5)
{
    #ifndef NoSync
      TIM_InitTypeDef  TIM_InitStructure;

      TIM_StructInit(&TIM_InitStructure);
      TIM_InitStructure.TIM_Mode = TIM_PWM;
      TIM_InitStructure.TIM_Clock_Source = TIM_CLK_EXTERNAL; // 10KHz
      TIM_InitStructure.TIM_Prescaler = 255; // Not Used
      TIM_InitStructure.TIM_Pulse_Level_1 = TIM_HIGH;
      TIM_InitStructure.TIM_Period_Level = TIM_LOW;
      //TIM_InitStructure.TIM_Full_Period = SyncOutPeriods[x24_SYNC_OUT_FPS-1];
      TIM_InitStructure.TIM_Full_Period = SyncOutPeriods[FramerateTime5-4];
      TIM_InitStructure.TIM_Pulse_Length_1 = TIM_InitStructure.TIM_Full_Period/2;
      TIM_Init (x24_SYNC_OUT_TIM, &TIM_InitStructure);
      TIM_CounterCmd(x24_SYNC_OUT_TIM, TIM_START); // Start the counter
    #endif
}

void StartSyncIn(void)
{
  #ifndef NoSync
   TIM_ITConfig(x24_SYNC_TIM, TIM_IT_TO | TIM_IT_IC1 | TIM_IT_IC2, ENABLE);
   TIM_CounterCmd(x24_SYNC_TIM, TIM_START); // Start the counter
  #endif
}

void StartTimSync(void)
{
  #ifndef NoSync
   TIM_ITConfig(x24_SYNC_TIM, TIM_IT_TO, ENABLE);
   TIM_CounterCmd(x24_SYNC_TIM, TIM_START); // Start the counter
  #endif
}

void Sync_HwConfig(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	TIM_InitTypeDef TIM_InitStructure;
	/* Configure Synchronization Peripherals */
#ifndef NoSync
	//  GPIO_StructInit(&GPIO_InitStructure);
	//  GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
	//  GPIO_InitStructure.GPIO_Pin = x24_SYNC_LED_GPIOx_Pin;
	//  GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
	//  GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
	//  GPIO_Init (x24_SYNC_LED_GPIO, &GPIO_InitStructure);

	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
	GPIO_InitStructure.GPIO_Pin = x24_SYNC_OUT_GPIOx_Pin;
	GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
	GPIO_InitStructure.GPIO_IPInputConnected = GPIO_IPInputConnected_Enable;
	GPIO_InitStructure.GPIO_Alternate = x24_SYNC_OUT_ALT; // PWM
	GPIO_Init (x24_SYNC_OUT_GPIO, &GPIO_InitStructure);

	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Direction = GPIO_PinInput;
	GPIO_InitStructure.GPIO_Pin = x24_SYNC_IN_GPIOx_Pin;
	GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
	GPIO_InitStructure.GPIO_IPInputConnected = GPIO_IPInputConnected_Enable;
	GPIO_InitStructure.GPIO_Alternate = x24_SYNC_IN_ALT; // ICAP1
	GPIO_Init (x24_SYNC_IN_GPIO, &GPIO_InitStructure);

	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Direction = GPIO_PinInput;
	GPIO_InitStructure.GPIO_Pin = x24_VSYNC_T_GPIOx_Pin;
	GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
	GPIO_InitStructure.GPIO_IPInputConnected = GPIO_IPInputConnected_Enable;
	GPIO_InitStructure.GPIO_Alternate = x24_VSYNC_T_ALT; // ICAP2
	GPIO_Init (x24_VSYNC_T_GPIO, &GPIO_InitStructure);

	TIM_StructInit(&TIM_InitStructure);
	TIM_InitStructure.TIM_Mode = TIM_PWM;
	TIM_InitStructure.TIM_Clock_Source = TIM_CLK_APB; // 48 MHz
	TIM_InitStructure.TIM_Prescaler = 47; // 1 MHz
	TIM_InitStructure.TIM_Pulse_Level_1 = TIM_HIGH;
	TIM_InitStructure.TIM_Period_Level = TIM_LOW;
	TIM_InitStructure.TIM_Full_Period = x24_SYNC_TIM_Res_us; // 1KHz
	TIM_InitStructure.TIM_Pulse_Length_1 = TIM_InitStructure.TIM_Full_Period/2;
	TIM_Init (x24_SYNC_TIM, &TIM_InitStructure);

	TIM_InitStructure.TIM_Mode = TIM_ICAP_CHANNEL_12;
	TIM_InitStructure.TIM_ICAP1_Edge = TIM_ICAP1_EDGE_RISING;
	TIM_InitStructure.TIM_ICAP2_Edge = TIM_ICAP2_EDGE_RISING; //TIM_ICAP2_EDGE_FALLING;
	TIM_Init (x24_SYNC_TIM, &TIM_InitStructure);

	Tcam1.OldGlobal=Tcam2.OldGlobal=0;
	Tcam1.CapPartial=Tcam2.CapPartial=0;
	Tcam1.CapGlobal=Tcam2.CapGlobal=0;
	//  TIM_ITConfig(x24_SYNC_TIM, TIM_IT_TO | TIM_IT_IC1 | TIM_IT_IC2, ENABLE);
	VIC_Config(x24_SYNC_TIM_ITx_LINE, VIC_IRQ, x24_SYNC_TIM_IPriority);
	VIC_ITCmd(x24_SYNC_TIM_ITx_LINE, ENABLE);

	//  VIC_Config(x24_SYNC_SW_IT_LINE, VIC_IRQ, x24_SW_IT_IPriority);
	//  VIC_ITCmd(x24_SYNC_SW_IT_LINE, ENABLE);

#endif
}
