/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
 * File Name          : DemoWebcam.c
 * Author             : AST Robotics group
 * Date First Issued  : 11 June 2007 : Version 1.0
 * Description        : This file implements the usb-webcam demo
 ********************************************************************************
 * History:
 * 11 June 2007 : Version 1.0
 *******************************************************************************
 THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS WITH
 CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
 AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
 OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
 OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
 CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
 *******************************************************************************/

/* Standard include ----------------------------------------------------------*/
#include "DemoBlobSearch.h"
#include "Demo.h"
#include "91x_map.h"
#include "utils.h"
#include "definitions.h"
#include "CamInt_x24.h"
#include "usb_lib.h"
#include "usb_conf.h"
#include "usb_prop.h"
#include "usb_pwr.h"
#include "usb_config.h"
#include <string.h>
#include "serial_protocol.h"
#include "RGBCube.h"
#include "blobsearch.h"
#include "math.h"

/* Include of other module interface headers ---------------------------------*/
/* Local includes ------------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define BUFFERSIZE_DEMO_BS	160*120*2

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
u8 UART_RxBuffer[UART_RxBufferSize];
TUart_Engine Uart_Engine;

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
void DemoBlobSearch(TMsg *Msg)
{


	u8 ImgBufferData[2*BUFFERSIZE_DEMO_BS];
	u8 RGBCubeData[4096];

	BufferSize = BUFFERSIZE_DEMO_BS;

	pImgBufferData = ImgBufferData;
	pRGBCubeData = RGBCubeData;

	RGBCube_Clear();
	BlobSearchInit();
	CamState=0;
	x24_WriteReg8(0x1380,0);  //wb
	x24_WriteReg8(0x1080,2);  //ae
	x24_WriteReg16(0x1095,1000);
	x24_WriteReg16(0x109d,240);
	x24_WriteRegF900(0x10a1,1.0);

	if(SenderInterface == SENDER_USB) {
		USB_SendMsg(Msg);
	} else {
		UART_SendMsg(Msg);
	}

	while(1) {
		Led_SM();
		CamJob_SM();
		if (USB_ReceivedMSG(Msg)) {
			if (Msg->Data[0]==DEV_ADDR) {
				if (HandleMSG(Msg)) {
					USB_SendMsg(Msg);
				}
			} else { // forward Msg to the other port
				UART_SendMsg(Msg);
			}
		}
		if (UART_ReceivedMSG(Msg)) {
			if (Msg->Data[0]==DEV_ADDR) {
				if (HandleMSG(Msg)) {
					UART_SendMsg(Msg);
				}
			} else { // forward Msg to the other port
				USB_SendMsg(Msg);
			}
		}
	}
}

int Handle_CMD_BlobSearchGetBlobs(TMsg *Msg)
{
	u32 i;
	u8 *pD;



	if (x24_Engine.Res_Colour_Type!=x24_ImageFormat_RGB_565) return 0;
	if (!CamJob_Pause()) return 0;
	Serialize(&Msg->Data[3], CompletedJob->FrameCount, 2);

	MY_ON();
	BlobSearch((u16*)CompletedJob->pFirstBlock->pData);
	MY_OFF();
	Msg->Data[5]=BlobCount;
	pD=&Msg->Data[6];

	u8 k;
	u8 myBlob;
	s16 maxArea;
	maxArea = Blobs[0].Area;
	myBlob = 0;
	for(k=0;k<BlobCount-1;k++){
		if(Blobs[k].Area < Blobs[k+1].Area){
			maxArea = Blobs[k+1].Area;
			myBlob = k+1;
		}
	}
	i = myBlob;
//	for (i=0; i<BlobCount; i++) {
		*pD=Blobs[i].BlobID; pD++;
		*pD=Blobs[i].ColorSetID; pD++;
		*pD=Blobs[i].Centroid.X; pD++;
		*pD=Blobs[i].Centroid.Y; pD++;
		*pD=Blobs[i].OuterBox.Top; pD++;
		*pD=Blobs[i].OuterBox.Bottom; pD++;
		*pD=Blobs[i].OuterBox.Left; pD++;
		*pD=Blobs[i].OuterBox.Right; pD++;
		Serialize(pD, Blobs[i].Perimeter, 2); pD+=2;
		Serialize(pD, Blobs[i].Area, 2); pD+=2;
		*pD=Blobs[i].Circularity; pD++;
//	}
	Msg->Len=13*BlobCount+6;
	return 1;
}

int Handle_CMD_FmiSaveBs(TMsg *Msg)
{
	FMI_WriteB1_Data(pRGBCubeData, 4096, 0, 0, 1);
	FMI_WriteB1_Data((u8 *)ColorSetParam, sizeof(TColorSetParam)*MAX_COLOR_SETS,
			0, 4096, 0);
	FMI_WriteB1_Data((u8 *)&SearchOptions, sizeof(SearchOptions), 0,
			4096+sizeof(TColorSetParam)*MAX_COLOR_SETS, 0);

	return 1;
}

int Handle_CMD_FmiLoadBs(TMsg *Msg)
{
	u8 supp[8], i;
	FMI_ReadB1_Data(pRGBCubeData, 4096, 0, 0);

	FMI_ReadB1_Data((u8 *)ColorSetParam, sizeof(TColorSetParam)*MAX_COLOR_SETS,
			0, 4096);

	FMI_ReadB1_Data(supp, sizeof(SearchOptions), 0,
			4096+sizeof(TColorSetParam)*MAX_COLOR_SETS);
	for(i=0;i<sizeof(SearchOptions);i++)
		((u8 *)&SearchOptions)[i] = supp[i];

	return 1;
}

/******************* (C) COPYRIGHT 2007 STMicroelectronics *****END OF FILE****/
