/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
* File Name          : DemoWebcam.c
* Author             : AST Robotics group
* Date First Issued  : 11 June 2007 : Version 1.0
* Description        : This file implements the usb-webcam demo
********************************************************************************
* History:
* 11 June 2007 : Version 1.0
*******************************************************************************
 THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS WITH
 CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
 AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
 OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
 OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
 CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

/* Standard include ----------------------------------------------------------*/
#include "DemoHarris.h"
#include "Demo.h"
#include "91x_map.h"
#include "utils.h"
#include "definitions.h"
#include "CamInt_x24.h"
#include "usb_lib.h"
#include "usb_conf.h"
#include "usb_prop.h"
#include "usb_pwr.h"
#include "usb_config.h"
#include <string.h>
#include "serial_protocol.h"
#include "math.h"
#include "harris_section.h"
/* Include of other module interface headers ---------------------------------*/
/* Local includes ------------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
#define BUFFERSIZE_DEMO_H	140*105
//4800
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

void DemoHarris(TMsg *Msg)
{
  ActiveDemo = DEMO_Harris;

  u8 ImgBuffer[2*BUFFERSIZE_DEMO_H];
  TCorner HarrisPoint[GlobalMaxHPt]; // Found Harris Points Data

  pImgBufferData = ImgBuffer;
  pFeature = HarrisPoint;

  BufferSize = BUFFERSIZE_DEMO_H;

  if(SenderInterface == SENDER_USB) {
	  USB_SendMsg(Msg);
  } else {
	  UART_SendMsg(Msg);
  }

  CamState=0;

  while(1) {
    Led_SM();
    CamJob_SM();
    if (USB_ReceivedMSG(Msg)) {
      if (Msg->Data[0]==DEV_ADDR) {
        if (HandleMSG(Msg)) {
          USB_SendMsg(Msg);
        }
      } else { // forward Msg to the other port
        UART_SendMsg(Msg);
      }
    }
    if (UART_ReceivedMSG(Msg)) {
      if (Msg->Data[0]==DEV_ADDR) {
        if (HandleMSG(Msg)) {
          UART_SendMsg(Msg);
        }
      } else { // forward Msg to the other port
        USB_SendMsg(Msg);
      }
    }
  }
}

void HarrisInitalization(void)
{
	u32 ImgCol = x24_Engine.Res_Width;

	//TMsg Msg;
	s32 MSImgData[SRow][SCol(MaxImgCol)];     // Smoothed image
	s32 MIx2Data[DerRow][DerCol(MaxImgCol)];  // Second order derivatives
	s32 MIy2Data[DerRow][DerCol(MaxImgCol)];
	s32 MIxyData[DerRow][DerCol(MaxImgCol)];
	s32 MsIx2Data[S2Row][S2Col(MaxImgCol)];   // Smoothed second order derivatives
	s32 MsIy2Data[S2Row][S2Col(MaxImgCol)];
	s32 MsIxyData[S2Row][S2Col(MaxImgCol)];
	s32 MHMData[S2Row][S2Col(MaxImgCol)];     // Harris Measure
	s32 MLM1Data[LMRow][LMCol(MaxImgCol)];    // 1st Local Maxima
	s32 MLM2Data[LMRow][LMCol(MaxImgCol)];    // 2nd Local Maxima
	//  s32 MSImgData[SRow][SCol];     // Smoothed image
	//  s32 MIx2Data[DerRow][DerCol];  // Second order derivatives
	//  s32 MIy2Data[DerRow][DerCol];
	//  s32 MIxyData[DerRow][DerCol];
	//  s32 MsIx2Data[S2Row][S2Col];   // Smoothed second order derivatives
	//  s32 MsIy2Data[S2Row][S2Col];
	//  s32 MsIxyData[S2Row][S2Col];
	//  s32 MHMData[S2Row][S2Col];     // Harris Measure
	//  s32 MLM1Data[LMRow][LMCol];    // 1st Local Maxima
	//  s32 MLM2Data[LMRow][LMCol];    // 2nd Local Maxima

	MatrixInit((TMatrix*)&MSImg, MSImgData, MSImgRows, SRow, SCol(ImgCol), 4);
	MatrixInit((TMatrix*)&MIx2, MIx2Data, MIx2Rows, DerRow, DerCol(ImgCol), 4);
	MatrixInit((TMatrix*)&MIy2, MIy2Data, MIy2Rows, DerRow, DerCol(ImgCol), 4);
	MatrixInit((TMatrix*)&MIxy, MIxyData, MIxyRows, DerRow, DerCol(ImgCol), 4);
	MatrixInit((TMatrix*)&MsIx2, MsIx2Data, MsIx2Rows, S2Row, S2Col(ImgCol), 4);
	MatrixInit((TMatrix*)&MsIy2, MsIy2Data, MsIy2Rows, S2Row, S2Col(ImgCol), 4);
	MatrixInit((TMatrix*)&MsIxy, MsIxyData, MsIxyRows, S2Row, S2Col(ImgCol), 4);
	MatrixInit((TMatrix*)&MHM, MHMData, MHMRows, S2Row, S2Col(ImgCol), 4);
	MatrixInit((TMatrix*)&MLM1, MLM1Data, MLM1Rows, LMRow, LMCol(ImgCol), 4);
	MatrixInit((TMatrix*)&MLM2, MLM2Data, MLM2Rows, LMRow, LMCol(ImgCol), 4);

	MaxHPt = GlobalMaxHPt;
	HarrisTh = 60;
	kh1_inv = 22;
}
#define MaxPtPerPacket 20
int Handle_CMD_GetHarrisPoints(TMsg *Msg)
{
  u32 i;
  u8 *pD;
  u16 PktNum=Msg->Data[3];
  u16 pointPerPacket;

  if(PktNum == 0) {

	  if (x24_Engine.Res_Colour_Type!=x24_ImageFormat_YCbCr_400) {
		  return 0;
	  } else {
		  YCbCrBug((u8 *)CompletedJob->pFirstBlock->pData, x24_Engine.Row_Size, (u16)CompletedJob->pFirstBlock->CurrentBlockRow);
	  }
	  if (!CamJob_Pause()) return 0;
	  Serialize(&Msg->Data[3], CompletedJob->FrameCount, 2);

	  HPtCount=0;
	  MY_ON();
	  Harris((u8*)CompletedJob->pFirstBlock->pData, x24_Engine.Res_Width, x24_Engine.Res_Height);
	  MY_OFF();
	  //  Serialize(&Msg->Data[7], x24_Engine.Res_Width, 2);
	  //      Serialize(&Msg->Data[9], x24_Engine.Res_Height, 2);
	  //      Serialize(&Msg->Data[11], DataLen, 2);
	  //      Serialize(&Msg->Data[13], CompletedJob->CurrImgRow*N, 2);

  }

  if(HPtCount>GlobalMaxHPt) HPtCount=GlobalMaxHPt;

  Msg->Data[5]=HPtCount;
  pD=&Msg->Data[6];

  if((HPtCount-(PktNum*MaxPtPerPacket))<MaxPtPerPacket)
	  pointPerPacket = HPtCount-(PktNum*MaxPtPerPacket);
  else
	  pointPerPacket = MaxPtPerPacket;

  for (i=PktNum*MaxPtPerPacket; i<HPtCount || i<(PktNum*MaxPtPerPacket+MaxPtPerPacket); i++) {
    *pD=pFeature[i].X; pD++;
    *pD=pFeature[i].Y; pD++;
    Serialize(pD, pFeature[i].Score, 2); pD+=2; // Harris Measure
  }
  Msg->Len=4*pointPerPacket+6;

  return 1;
}

int Handle_CMD_HarrisSettings(TMsg *Msg)
{
  u32 i;
  u8 *pD;

  MaxHPt = Deserialize(&Msg->Data[3], 2);
  HarrisTh = Deserialize(&Msg->Data[5], 2);
  kh1_inv = Deserialize(&Msg->Data[7], 2);
  if(MaxHPt>GlobalMaxHPt) {
	  MaxHPt=GlobalMaxHPt;
  }
  Msg->Len=3;
  return 1;
}

/******************* (C) COPYRIGHT 2007 STMicroelectronics *****END OF FILE****/
