/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
* File Name          : fmi.c
* Author             : AST Robotics group
* Date First Issued  : 6 May 2010 : Version 1.0
* Description        :
********************************************************************************
* History:
* 6 May 2010 : Version 1.0
*******************************************************************************
 THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS WITH
 CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
 AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
 OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
 OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
 CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/


/* Standard include ----------------------------------------------------------*/
#include "91x_map.h"
#include "91x_fmi.h"
#include "fmi.h"
//#include "definitions.h"
//#include "definitionsLib.h"

/* Include of other module interface headers ---------------------------------*/
/* Local includes ------------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/*******************************************************************************
* Function Name  : FMI_Configuration
* Description    : Configures the Flash Memory Inferface
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void FMI_Configuration(void)
{
#ifdef Boot_Bank_0
   FMI_BankRemapConfig(0x4,0x2,0x0,0x80000);
#else
   // 32K, 512,
   FMI_BankRemapConfig(0x0,0x6,0x0,0x80000);
#endif
}

/*******************************************************************************
* Function Name  : FMI_WriteB1S0_U32Data
* Description    : Write u32 data in Flash (Bank 1 Sector 0)
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void FMI_WriteB1S0_U32Data(u32 data)
{
//    u8 FMI_Timeout_Status=0;
    FMI_WriteProtectionCmd(FMI_B1S0, DISABLE);
    FMI_EraseSector(FMI_B1S0);
    FMI_WaitForLastOperation(FMI_BANK_1);

    FMI_WriteHalfWord(0x80000,(data & 0xFFFF));
    FMI_WaitForLastOperation(FMI_BANK_1);
    FMI_WriteHalfWord(0x80002,(data>>16));
    FMI_WaitForLastOperation(FMI_BANK_1);
}

//void FMI_WriteB0_Buffer(u8 *data, u32 Len, u32 Sector)
//{
//  int i=0;
////  u32 Len32 = Len/4;
////  u32 Mod32 = fmod(Len,4);
//  u32 addr = BANK0_OFFSET + Sector*0x10000;
//  FMI_WriteProtectionCmd(Sector*0x00010000 + FMI_BANK_0, DISABLE);
//
//  FMI_EraseSector(Sector*0x00010000 + FMI_BANK_0);
//  FMI_WaitForLastOperation(FMI_BANK_0);
//
//  while(i<Len) {
//    FMI_WriteHalfWord(addr,(data[i]+(data[i+1]<<8))& 0xFFFF);
//    FMI_WaitForLastOperation(FMI_BANK_0);
//    addr+=2;
//    i+=2;
//    FMI_WriteHalfWord(addr,(data[i]+(data[i+1]<<8))& 0xFFFF);
//    FMI_WaitForLastOperation(FMI_BANK_0);
//    addr+=2;
//    i+=2;
//  }
//}
//
//void FMI_ReadB0_Buffer(u8 *data, u32 Len, u8 Sector)
//{
//  u32 i;
//  u32 Len32 = floor(Len/4);
//  u32 Mod32 = fmod(Len,4);
//  u32 data32;
//
//  u32 *addr = (u32*)(BANK0_OFFSET + Sector*0x10000);
//
//  for(i=0;i<Len32;i++) {
//    data32 = *addr;
//    *data = (data32&0xFF);
//    data++;
//    *data = ((data32>>8)&0xFF);
//    data++;
//    *data = ((data32>>16)&0xFF);
//    data++;
//    *data = ((data32>>24)&0xFF);
//    data++;
//    addr++;
//  }
//  if(!Mod32) return;
//  data32 = *addr;
//  for(i=0;i<Mod32;i++) {
//    *data = ((data32>>(8*i))&0xFF);
//    data++;
//  }
//}

void FMI_WriteB1_Data(u8 *data, u32 Len, u8 Sector, u32 addrOffset, u8 erase)
{
	int i=0;
	//  u32 Len32 = Len/4;
	//  u32 Mod32 = fmod(Len,4);
	u32 addr = 0x80000 + Sector*0x2000 + addrOffset;
	if(erase) {
		FMI_WriteProtectionCmd(Sector*0x00002000 + FMI_BANK_1, DISABLE);

		FMI_EraseSector(Sector*0x00002000 + FMI_BANK_1);
		FMI_WaitForLastOperation(FMI_BANK_1);
	}
	while(i<Len) {
		FMI_WriteHalfWord(addr,(data[i]+(data[i+1]<<8))& 0xFFFF);
		FMI_WaitForLastOperation(FMI_BANK_1);
		addr+=2;
		i+=2;
		FMI_WriteHalfWord(addr,(data[i]+(data[i+1]<<8))& 0xFFFF);
		FMI_WaitForLastOperation(FMI_BANK_1);
		addr+=2;
		i+=2;
	}
}

void FMI_ReadB1_Data(u8 *data, u32 Len, u8 Sector, u32 addrOffset)
{
  u32 i;
  u32 Len32 = floor(Len/4);
  u32 Mod32 = fmod(Len,4);
  u32 data32;

  u32 *addr = (u32*)(0x80000 + Sector*0x2000 + addrOffset);

  for(i=0;i<Len32;i++) {
    data32 = *addr;
    *data = (data32&0xFF);
    data++;
    *data = ((data32>>8)&0xFF);
    data++;
    *data = ((data32>>16)&0xFF);
    data++;
    *data = ((data32>>24)&0xFF);
    data++;
    addr++;
  }
  if(!Mod32) return;
  data32 = *addr;
  for(i=0;i<Mod32;i++) {
    *data = ((data32>>(8*i))&0xFF);
    data++;
  }
}
