/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
* File Name          : main.c
* Author             : AST Robotics group
* Date First Issued  : 11 May 2007 : Version 1.0
* Description        : Main program body
********************************************************************************
* History:
* 28 May 2007 : Version 1.2
* 11 May 2007 : Version 1.0
********************************************************************************
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "91x_lib.h"
#include "definitions.h"
#include "CamInt_x24.h"
#include "utils.h"
#include "math.h"
#include "usb_lib.h"
#include "usb_conf.h"
#include "usb_prop.h"
#include "usb_pwr.h"
#include "usb_config.h"
#include "Demo.h"
#include "fmi.h"
#include "definitionsLib.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/

//void CAMCLK_Configuration(void);
void SCU_Configuration(void);
void USB_Configuration(void);
void GPIO_Configuration(void);
void UART_Configuration(void);
void I2C0_Config_Default(void);
void Uart_Dma_Configuration(void);

/* Private functions ---------------------------------------------------------*/

/*******************************************************************************
* Function Name  : main
* Description    : Main program
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
int main(void)
{
  TMsg Msg;
  u16 i=0;

  #ifdef DEBUG
    debug();
  #endif

  /* Configure the system clocks */
  SCU_Configuration();

  /* Configure the FMI */
  FMI_Configuration();

  /* Configure the GPIO ports */
  GPIO_Configuration();

  /* Configure x24 Camera */
  x24_HwConfig(8000000);

//  CAMCLK_Configuration();
  Sync_HwConfig();

  x24_SendPatches();

  /* Configure the UARTS */
  UART_Configuration();

  /* Configure the USB */
  USB_Configuration();
  USB_Init();

  LED_ON(LEDR); MSDelay(200); LED_OFF(LEDR); MSDelay(200);
  LED_ON(LEDR); MSDelay(200); LED_OFF(LEDR); MSDelay(200);
  LED_ON(LEDR); MSDelay(200); LED_OFF(LEDR);

//  UART_SendString(UartNum, "hello world\r\n");
//  USB_WriteString("Hello World BIS\r\n");

  while(1) {
	  Led_SM();
	  CamJob_SM();
	  if (USB_ReceivedMSG(&Msg)) {
		  SenderInterface = SENDER_USB;
		  if (Msg.Data[0]==DEV_ADDR) {
			  if (HandleMSG(&Msg)) {
				  USB_SendMsg(&Msg);
			  }
		  } else { // forward Msg to the other port
			  UART_SendMsg(&Msg);
		  }
	  }
	  if (UART_ReceivedMSG(&Msg)) {
		  SenderInterface = SENDER_UART;
		  if (Msg.Data[0]==DEV_ADDR) {
			  if (HandleMSG(&Msg)) {
				  UART_SendMsg(&Msg);
			  }
		  } else { // forward Msg to the other port
			  USB_SendMsg(&Msg);
		  }
	  }
  }
}


/*******************************************************************************
* Function Name  : SCU_Configuration
* Description    : Configures the system clocks.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void SCU_Configuration(void)
{
  /* Initialize PLL */
  SCU_MCLKSourceConfig(SCU_MCLK_OSC);
  SCU_FMICLKDivisorConfig(SCU_FMICLK_Div1);
  FMI_Config(FMI_READ_WAIT_STATE_2,FMI_WRITE_WAIT_STATE_0, FMI_PWD_ENABLE,\
             FMI_LVD_ENABLE,FMI_FREQ_HIGH);
  SCU_RCLKDivisorConfig(SCU_RCLK_Div1);
  SCU_HCLKDivisorConfig(SCU_HCLK_Div1);
  SCU_PCLKDivisorConfig(SCU_PCLK_Div2);
  SCU_BRCLKDivisorConfig(SCU_BRCLK_Div1);
  SCU_PLLFactorsConfig(192,25,2);            /* PLL = 96 MHz */
  SCU_PLLCmd(ENABLE);                        /* PLL Enabled  */
  SCU_MCLKSourceConfig(SCU_MCLK_PLL);        /* MCLK = PLL   */

  SCU_PFQBCCmd(ENABLE);

  /* Enable GPIO 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 Clocks */
  SCU_APBPeriphClockConfig(__GPIO0, ENABLE);
  GPIO_DeInit(GPIO0);
  SCU_APBPeriphClockConfig(__GPIO1, ENABLE);
  GPIO_DeInit(GPIO1);
  SCU_APBPeriphClockConfig(__GPIO2, ENABLE);
  GPIO_DeInit(GPIO2);
  SCU_APBPeriphClockConfig(__GPIO3, ENABLE);
  GPIO_DeInit(GPIO3);
  SCU_APBPeriphClockConfig(__GPIO4, ENABLE);
  GPIO_DeInit(GPIO4);
  SCU_APBPeriphClockConfig(__GPIO5, ENABLE);
  GPIO_DeInit(GPIO5);
  SCU_APBPeriphClockConfig(__GPIO6, ENABLE);
  GPIO_DeInit(GPIO6);
  SCU_APBPeriphClockConfig(__GPIO7, ENABLE);
  GPIO_DeInit(GPIO7);
  SCU_APBPeriphClockConfig(__GPIO8, ENABLE);
  GPIO_DeInit(GPIO8);
  SCU_APBPeriphClockConfig(__GPIO9, ENABLE);
  GPIO_DeInit(GPIO9);

  /* Enable UART Clock */
#ifndef NoUart
  SCU_APBPeriphClockConfig(__UartNum, ENABLE);
  UART_DeInit(UartNum);
#endif

  /* Enable VIC clock */
  SCU_AHBPeriphClockConfig(__VIC, ENABLE);
  VIC_DeInit();

  /* Enable WIU clock */
  SCU_APBPeriphClockConfig(__WIU, ENABLE);
  WIU_DeInit();

  /* Enable WIU clock */
  SCU_APBPeriphClockConfig(__I2C0, ENABLE);

  /* Enable DMA clock */
  SCU_AHBPeriphClockConfig(__DMA, ENABLE);
  DMA_DeInit();

  /* Enable TIM0123 clock */
  SCU_APBPeriphClockConfig(__TIM01, ENABLE);
  TIM_DeInit(TIM0);
  TIM_DeInit(TIM1);
  SCU_APBPeriphClockConfig(__TIM23, ENABLE);
  TIM_DeInit(TIM2);
  TIM_DeInit(TIM3);
  SCU_TIMPresConfig(SCU_TIM01, 4800); // ~10KHz
  SCU_TIMExtCLKCmd(SCU_TIM01, DISABLE); // Disable external pin
  SCU_TIMPresConfig(SCU_TIM23, 4800); // ~10KHz
  SCU_TIMExtCLKCmd(SCU_TIM23, DISABLE); // Disable external pin

  SCU_APBPeriphClockConfig(__I2C0, ENABLE);
  I2C_DeInit(I2C0);

  SCU_AHBPeriphClockConfig(__FMI, ENABLE);
  SCU_AHBPeriphReset(__FMI, DISABLE);
//  SCU_TIMCLKSourceConfig(SCU_TIM01, SCU_TIMCLK_INT);
//  SCU_TIMCLKSourceConfig(SCU_TIM23, SCU_TIMCLK_INT);
}

/*******************************************************************************
* Function Name  : USB_Configuration
* Description    : Configures the USB
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void USB_Configuration(void)
{
#ifndef DemoUART
  GPIO_InitTypeDef GPIO_InitStructure;

  /* USB clock = MCLK/2 = 48MHz */
  SCU_USBCLKConfig(SCU_USBCLK_MCLK2);
  /* Enable USB clock */
  SCU_AHBPeriphClockConfig(__USB, ENABLE);
  SCU_AHBPeriphReset(__USB, DISABLE);
  SCU_AHBPeriphClockConfig(__USB48M, ENABLE);

  /* Configure USB D+ PullUp pin */
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
  GPIO_InitStructure.GPIO_Pin = USB_Dp_PullUp_GPIOx_Pin;
  GPIO_InitStructure.GPIO_Type = GPIO_Type_OpenCollector;//GPIO_Type_PushPull;
  GPIO_InitStructure.GPIO_Alternate=GPIO_OutputAlt1;
  GPIO_Init (USB_Dp_PullUp_GPIO, &GPIO_InitStructure);
  USB_Dp_PullUp_OFF();
  MSDelay(100);
  USB_Dp_PullUp_ON();

  USB_endPointConf();

  VIC_Config(USBLP_ITLine, VIC_IRQ, USB_Priority);
  VIC_ITCmd(USBLP_ITLine, ENABLE);
#endif
}

/*******************************************************************************
* Function Name  : GPIO_Configuration
* Description    : Configures the different GPIO ports.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void GPIO_Configuration(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

#ifndef NoUart
  /* Configure UART RxD */
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Direction = GPIO_PinInput;
  GPIO_InitStructure.GPIO_Pin = UART_RX_GPIOx_Pin;
  GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull ;
  GPIO_InitStructure.GPIO_Alternate = GPIO_InputAlt1;
  GPIO_InitStructure.GPIO_IPInputConnected = GPIO_IPInputConnected_Enable;
  GPIO_Init (UART_RX_GPIO, &GPIO_InitStructure);

  /* Configure UART TxD */
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
  GPIO_InitStructure.GPIO_Pin = UART_TX_GPIOx_Pin;
  GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull ;
  GPIO_InitStructure.GPIO_Alternate = UART_TX_GPIOx_OutAlt;
  GPIO_Init (UART_TX_GPIO, &GPIO_InitStructure);

  /* Configure UART DE */
  DE_OFF(DE);
  GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
  GPIO_InitStructure.GPIO_Pin = DE_GPIOx_Pin;
  GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
  GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
  GPIO_Init (DE_GPIO, &GPIO_InitStructure);

#endif

  /* Configure LEDR */
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
  GPIO_InitStructure.GPIO_Pin = LEDR_GPIOx_Pin;
  GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull ;
  GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
  GPIO_Init (LEDR_GPIO, &GPIO_InitStructure);

  /* Configure MY_GPIO */
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
  GPIO_InitStructure.GPIO_Pin = MY_GPIOx_Pin;
  GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull ;
  GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1;
  GPIO_Init (MY_GPIO, &GPIO_InitStructure);
 }

/*******************************************************************************
* Function Name  : UART_Configuration
* Description    : Configures the different UARTS
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void Uart_Dma_Configuration(void)
{
  DMA_InitTypeDef  DMA_InitStruct;

  DMA_DeInit();
  DMA_Cmd(ENABLE);
  DMA_StructInit(&DMA_InitStruct);
  DMA_InitStruct.DMA_Channel_LLstItm=(u32)&Uart_Engine.DMA_LLI.SrcAdd;
  DMA_InitStruct.DMA_Channel_SrcAdd=(u32)(&UartNum->DR);
  DMA_InitStruct.DMA_Channel_DesAdd=(u32)(&UART_RxBuffer[0]);
  DMA_InitStruct.DMA_Channel_SrcWidth= DMA_SrcWidth_Byte;
  DMA_InitStruct.DMA_Channel_DesWidth= DMA_DesWidth_Byte;
  DMA_InitStruct.DMA_Channel_SrcBstSize = DMA_SrcBst_1Data ;
  DMA_InitStruct.DMA_Channel_FlowCntrl=DMA_FlowCntrl2_DMA;
  DMA_InitStruct.DMA_Channel_Src= DMA_SRC_UARTx_RX;
  DMA_InitStruct.DMA_Channel_TrsfSize = UART_RxBufferSize;
//  DMA_ITConfig(DMA_Channel1, ENABLE); // Enable terminal count interrupt
//  DMA_ITMaskConfig(DMA_Channel1, DMA_ITMask_ITC, ENABLE); // Enable terminal count interrupt
  DMA_ChannelDESIncConfig (UART_DMA_Channelx, ENABLE);
  DMA_Init(UART_DMA_Channelx, &DMA_InitStruct);

  Uart_Engine.DMA_LLI.SrcAdd=(u32)(&UartNum->DR);
  Uart_Engine.DMA_LLI.DesAdd=(u32)(&UART_RxBuffer[0]);
  Uart_Engine.DMA_LLI.CCR = UART_DMA_Channelx->CC & 0xFFFFF000;
  Uart_Engine.DMA_LLI.CCR |= (DMA_InitStruct.DMA_Channel_TrsfSize & 0x0FFF);
  Uart_Engine.DMA_LLI.LLstItm=(u32)&Uart_Engine.DMA_LLI.SrcAdd;

  Uart_Engine.StartOfMsg = 0;

  DMA_ChannelCmd (UART_DMA_Channelx,ENABLE);
}

/*******************************************************************************
* Function Name  : UART_Configuration
* Description    : Configures the different UARTS
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/

void UART_Configuration(void)
{
#ifndef NoUart
  UART_InitTypeDef UART_InitStructure;
  UART_InitStructure.UART_WordLength = UART_WordLength_8D;
  UART_InitStructure.UART_StopBits = UART_StopBits_1;
  UART_InitStructure.UART_Parity = UART_Parity_No ;
  UART_InitStructure.UART_BaudRate = UartBoudRate;
  UART_InitStructure.UART_HardwareFlowControl = UART_HardwareFlowControl_None;
  UART_InitStructure.UART_Mode = UART_Mode_Tx_Rx;
  UART_InitStructure.UART_FIFO = UART_FIFO_Enable;
  UART_InitStructure.UART_TxFIFOLevel = UART_FIFOLevel_1_2; /* FIFO size 16 bytes, FIFO level 8 bytes */
  UART_InitStructure.UART_RxFIFOLevel = UART_FIFOLevel_1_2; /* FIFO size 16 bytes, FIFO level 8 bytes */
  UART_Init(UartNum, &UART_InitStructure);
  UART_Cmd(UartNum, ENABLE);

   /* Enable the DMA request for the Rx FIFO */
  UART_DMACmd(UartNum,  UART_DMAReq_Rx , ENABLE);
  DMA_SetBReq(DMA_UARTx_RX_Mask);
  UART_Cmd(UartNum, ENABLE);

  Uart_Engine.StartOfMsg = 0;
  Uart_Dma_Configuration();
#endif
}

/******************* (C) COPYRIGHT 2007 STMicroelectronics *****END OF FILE****/
