/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
* File Name          : utils.c
* Author             : AST Robotics group 
* Date First Issued  : 11 May 2007 : Version 1.0
* Description        : This file provides a set of functions for
*                      general purpose use.
********************************************************************************
* History:
* 28 May 2007 : Version 1.2
* 11 May 2007 : Version 1.0
*******************************************************************************
 THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS WITH
 CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
 AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
 OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
 OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
 CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

/* Standard include ----------------------------------------------------------*/
#include "utils.h"
#include "math.h"

/* Include of other module interface headers ---------------------------------*/
/* Local includes ------------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/*******************************************************************************
* Function Name  : uDelay
* Description    : Inserts a delay time of nCount Micro Seconds.
* Input          : nCount: specifies the amount of delay in Micro Seconds
* Output         : None
* Return         : None
*******************************************************************************/
void uDelay(u32 nCount)
{
  u32 i;

  for(i = nCount; i != 0; i--) { // 89 nop
    asm("nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n"
        "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" 
        "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" 
        "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" 
        "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" 
        "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" 
        "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" 
        "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" 
        "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n" "nop\n");
  }
}

/*******************************************************************************
* Function Name  : MSDelay
* Description    : Inserts a delay time of nCount Milli Seconds.
* Input          : nCount: specifies the amount of delay in Milli Seconds
* Output         : None
* Return         : None
*******************************************************************************/
void MSDelay(u32 nCount)
{
  u32 i;

  for(i = nCount; i != 0; i--)
    uDelay(1000);
}

/*******************************************************************************
* Function Name  : UART_SendByte
* Description    : Transmits a Byte through the UARTx peripheral.
* Input          : - UARTx: where x can be 0,1 or 2 to select the UART peripheral.
*                  - Data: the byte to transmit
* Output         : None
* Return         : None
*******************************************************************************/
void UART_SendByte(UART_TypeDef* UARTx, u8 Data)
{
  while(UART_GetFlagStatus(UARTx, UART_FLAG_TxFIFOFull) != RESET);
  UARTx->DR = Data;
}

/*******************************************************************************
* Function Name  : UART_SendString
* Description    : Transmits a null terminated string through the UARTx peripheral.
* Input          : - UARTx: where x can be 0,1 or 2 to select the UART peripheral.
*                  - String: the null terminated string to transmit
* Output         : None
* Return         : None
*******************************************************************************/
void UART_SendString(UART_TypeDef* UARTx, u8 *String)
{
  while ((*String)!='\0') {
    UART_SendByte(UARTx, *String); // Transmit one byte
    String++;
  }
}

/*******************************************************************************
* Function Name  : UART_SendBuffer
* Description    : Transmits a Buffer through the UARTx peripheral.
* Input          : - UARTx: where x can be 0,1 or 2 to select the UART peripheral.
*                  - pBuf: the Buffer to transmit
*                  - Len: Lenght of pBuf
* Output         : None
* Return         : None
*******************************************************************************/
void UART_SendBuffer(UART_TypeDef* UARTx, u8 *pBuf, u16 Len)
{
  while (Len>0) {
    UART_SendByte(UARTx, *pBuf); // Transmit one byte
    pBuf++;
    Len--;
  }
}

/*******************************************************************************
* Function Name  : UART_SendBufferASHII
* Description    : Transmits a Buffer through the UARTx peripheral.
*                  the Buffer is converted in ashii on the fly
* Input          : - UARTx: where x can be 0,1 or 2 to select the UART peripheral.
*                  - pBuf: the Buffer to transmit
*                  - Len: Lenght of pBuf
* Output         : None
* Return         : None
*******************************************************************************/
void UART_SendBufferASHII(UART_TypeDef* UARTx, u8 *pBuf, u16 Len)
{
  while (Len>0) {
    while(UART_GetFlagStatus(UARTx, UART_FLAG_TxFIFOFull) != RESET);
    UARTx->DR = u4toAshii(*pBuf & 0x0F); // Transmit first nibble
    while(UART_GetFlagStatus(UARTx, UART_FLAG_TxFIFOFull) != RESET);
    UARTx->DR = u4toAshii(*pBuf>>4); // Transmit second nibble
    pBuf++;
    Len--;
  }
}

/*******************************************************************************
* Function Name  : Num2String
* Description    : Convert a Number to a null terminetd string (ashii)
* Input          : - Num: Number to be converted
* Output         : - String: the output null terminated string
* Return         : None
*******************************************************************************/
void StrAddChar(u8 c, u8 *String)
{
  u8 i, c1;
  
  i=0;
  do {
    c1=String[i];
    String[i]=c;
    c=c1;
    i++;
  } while (String[i-1]!=0);
}

void Num2String(int Num, u8 *String)
{
  u8 r, Neg;
  
  String[0]=0;
  Neg=0;
  if (Num<0) {
    Neg=1;
    Num=-Num;
  }
  do {
    r=Num%10;
    Num/=10;
    StrAddChar('0'+r, String);
  } while (Num>0);
  if (Neg) StrAddChar('-', String);
}

/*******************************************************************************
* Function Name  : u4toAshii
* Description    : Convert the 4 LSB of a char to its ashii code
* Input          : - ch: Char to be converted
* Output         : None 
* Return         : The ashii code of ch
*******************************************************************************/
u8 u4toAshii(u8 ch)
{
  if (ch < 10) {
    return ch + '0';
  } else {
    return ch + ('A' - 10);
  }
}

/*******************************************************************************
* Function Name  : u8toDoubleAshii
* Description    : Convert a char to two chars, corresponding to the ashii code
* Input          : - ch: Char to be converted
* Output         : - Ashii: the two byte ashii code of ch
* Return         : None
*******************************************************************************/
void u8toDoubleAshii(u8 ch, u8 *Ashii)
{
  Ashii[0]=u4toAshii(ch & 0x0F);
  Ashii[1]=u4toAshii(ch >>4);
}

/*******************************************************************************
* Function Name  : u16toQuadAshii
* Description    : Convert a char to four chars, corresponding to the ashii code
* Input          : - i: the u16 to be converted
* Output         : - Ashii: the four byte ashii code of i
* Return         : None
*******************************************************************************/
void u16toQuadAshii(u16 i, u8 *Ashii)
{
  u8 Ch;
  Ch=i & 0xff;
  u8toDoubleAshii(Ch, Ashii);
  Ch=(i>>8) & 0xff;
  u8toDoubleAshii(Ch, &Ashii[2]);
}

/*******************************************************************************
* Function Name  : u32toOctAshii
* Description    : Convert a char to eight chars, corresponding to the ashii code
* Input          : - i: the u32 to be converted
* Output         : - Ashii: the eight byte ashii code of i
* Return         : None
*******************************************************************************/
void u32toOctAshii(u32 i, u8 *Ashii)
{
  u8 Ch;
  Ch=i & 0xff;
  u8toDoubleAshii(Ch, Ashii);
  Ch=(i>>=8) & 0xff;
  u8toDoubleAshii(Ch, &Ashii[2]);
  Ch=(i>>=8) & 0xff;
  u8toDoubleAshii(Ch, &Ashii[4]);
  Ch=(i>>=8) & 0xff;
  u8toDoubleAshii(Ch, &Ashii[6]);
}

/*******************************************************************************
* Function Name  : Ashii_to_Unsigned
* Description    : Convert a ashii code string into a Unsigned Integer
* Input          : - Ashii: ashii code string to be converted
*                  - N: Lenght of the string
* Output         : None
* Return         : The Integer represented by the string
*******************************************************************************/
u8 Ashii_to_u4(u8 Ashii)
{
  if (Ashii > '9')
    return Ashii-'A'+10;
  else
    return Ashii-'0';
}

u32 Ashii_to_Unsigned(u8 *Ashii, u8 N)
{
  u8 i;
  u32 app;
  
  app=0;
  for (i=0; i<N; i++) {
    app+=Ashii_to_u4(Ashii[i])<<(4*i);
  }
  return app;
}

/*******************************************************************************
* Function Name  : StrLen
* Description    : Compute the lenght of the string
* Input          : the string
* Output         : none
* Return         : the lenght
*******************************************************************************/
u32 StrLen(u8 *Str)
{
  u32 i;
  for (i=0; Str[i]!='\0'; i++);
  return i;
}

/******************* (C) COPYRIGHT 2007 STMicroelectronics *****END OF FILE****/
